@@ -85,7 +85,8 @@ class ArithmeticTestCase(UflTestCase):
     def test_elem_op(self):
         x, y, z = tetrahedron.x
         A = as_matrix(((x,y,z),(3,4,5)))
-        self.assertEqual(elem_op(sin, A), as_matrix(((sin(x),sin(y),sin(z)),(sin(3),sin(4),sin(5)))))
+        self.assertEqual(elem_op(sin, A), as_matrix(((sin(x),sin(y),sin(z)),
+                                                     (sin(3),sin(4),sin(5)))))
         self.assertEqual(elem_op(sin, A).dx(0).shape(), (2, 3))
 
 if __name__ == "__main__":
