@@ -11,7 +11,15 @@ from ufl.constantvalue import as_ufl
 from ufl.classes import * 
 from ufl.algorithms import * 
 
+has_repr = set()
+has_dict = set()
 def test_object(a, shape, free_indices):
+    # Check if instances of this type has certain memory consuming members
+    if hasattr(a, '_repr'):
+        has_repr.add(a.__class__.__name__)
+    if hasattr(a, '__dict__'):
+        has_dict.add(a.__class__.__name__)
+
     # Test reproduction via repr string
     r = repr(a)
     e = eval(r, globals())
@@ -35,6 +43,12 @@ def test_object(a, shape, free_indices):
         assert sh == shape
 
 def test_object2(a):
+    # Check if instances of this type has certain memory consuming members
+    if hasattr(a, '_repr'):
+        has_repr.add(a.__class__.__name__)
+    if hasattr(a, '__dict__'):
+        has_dict.add(a.__class__.__name__)
+
     # Test reproduction via repr string
     r = repr(a)
     e = eval(r, globals())
@@ -129,7 +143,7 @@ class ClasscoverageTest(UflTestCase):
         test_object(a, (), ())
         
         I = Identity(1)
-        test_object(I, (), ())
+        test_object(I, (1,1), ())
         I = Identity(2)
         test_object(I, (2,2), ())
         I = Identity(3)
@@ -195,10 +209,10 @@ class ClasscoverageTest(UflTestCase):
             a = inner(v, f)
             test_object(a, None, None)
         
-        for v in (v1,v2,v3):
-            for f in (f1,f2,f3):
-                a = dot(v, f)
-                test_object(a, None, None)
+        for v,f in zip((v1,v2,v3),(f1,f2,f3)):
+            a = dot(v, f)
+            sh = v.shape()[:-1] + f.shape()[1:]
+            test_object(a, sh, None)
         
         a = cross(v13D, f13D)
         test_object(a, (3,), ())
@@ -331,6 +345,12 @@ class ClasscoverageTest(UflTestCase):
         test_object(a, (), ())
         a = tan(f0)
         test_object(a, (), ())
+        a = cosh(f0)
+        test_object(a, (), ())
+        a = sinh(f0)
+        test_object(a, (), ())
+        a = tanh(f0)
+        test_object(a, (), ())
         a = exp(f0)
         test_object(a, (), ())
         a = ln(f0)
@@ -353,6 +373,12 @@ class ClasscoverageTest(UflTestCase):
         test_object(a, (), ())
         a = Tan(one)
         test_object(a, (), ())
+        a = Cosh(one)
+        test_object(a, (), ())
+        a = Sinh(one)
+        test_object(a, (), ())
+        a = Tanh(one)
+        test_object(a, (), ())
         a = Acos(one)
         test_object(a, (), ())
         a = Asin(one)
@@ -448,19 +474,32 @@ class ClasscoverageTest(UflTestCase):
         test_object(a, (), ())
         
         #a = PositiveRestricted(v0)
-        test_object(a, (), ())
+        #test_object(a, (), ())
         a = v0('+')
         test_object(a, (), ())
         a = v0('+')*f0
         test_object(a, (), ())
         
         #a = NegativeRestricted(v0)
-        test_object(a, (), ())
+        #test_object(a, (), ())
         a = v0('-')
         test_object(a, (), ())
         a = v0('-') + f0
         test_object(a, (), ())
 
+        a = cell_avg(v0)
+        test_object(a, (), ())
+        a = facet_avg(v0)
+        test_object(a, (), ())
+        a = cell_avg(v1)
+        test_object(a, (dim,), ())
+        a = facet_avg(v1)
+        test_object(a, (dim,), ())
+        a = cell_avg(v1)[i]
+        test_object(a, (), (i,))
+        a = facet_avg(v1)[i]
+        test_object(a, (), (i,))
+
         # --- Integrals:
 
         a = v0*dx
@@ -510,22 +549,41 @@ class ClasscoverageTest(UflTestCase):
         f = action(d)
         #e = action(b)
 
-        # TODO: Add tests for TensorConstant, VectorConstant, ScalarSomething, Skew
-
         # --- Check which classes have been created
-        if ufl.expr._class_usage_statistics:
-            s = ufl.expr._class_usage_statistics
+        if Expr._class_usage_statistics:
+            s = Expr._class_usage_statistics
             constructed = set(s.keys())
             abstract = set((Expr, Terminal, Operator, FormArgument, ConstantBase, AlgebraOperator,
-                            Condition, BinaryCondition, MathFunction, Restricted, ScalarValue,
-                            ConstantValue, CompoundDerivative, Derivative,
+                            Condition, BinaryCondition, MathFunction, BesselFunction, Restricted, ScalarValue,
+                            ConstantValue, IndexAnnotated, CompoundDerivative, Derivative,
                             WrapperType, GeometricQuantity, CompoundTensorOperator, UtilityType))
             unused = set(ufl.classes.all_ufl_classes) - constructed - abstract
             if unused:
-                print 
+                print
                 print "The following classes were never instantiated in class coverage test:"
                 print "\n".join(sorted(map(str,unused)))
-                print 
+                print
+        # --- Check which classes had certain member variables
+        if has_repr:
+            print
+            print "The following classes contain a _repr member:"
+            print "\n".join(sorted(map(str,has_repr)))
+            print
+        if has_dict:
+            print
+            print "The following classes contain a __dict__ member:"
+            print "\n".join(sorted(map(str,has_dict)))
+            print
+
+        # TODO: Add tests for bessel functions:
+        #   BesselI
+        #   BesselJ
+        #   BesselK
+        #   BesselY
+        #   Erf
+        # TODO: Add tests for:
+        #   Label
+
 
 if __name__ == "__main__":
     main()
