@@ -174,6 +174,24 @@ class DerivativeTestCase(UflTestCase):
         def df(w, v): return sign(w)*v
         self._test(f, df)
 
+    def testConditional(self):
+        def cond(w): return lt(1.0, 2.0)
+        def f(w):     return conditional(cond(w), 2*w, 3*w)
+        def df(w, v): return 2*v
+        self._test(f, df)
+
+        def cond(w): return lt(2.0, 1.0)
+        def f(w):     return conditional(cond(w), 2*w, 3*w)
+        def df(w, v): return 3*v
+        self._test(f, df)
+
+    def testConditional(self): # This will fail without bugfix in derivative
+        def cond(w): return lt(w, 1.0)
+        def f(w):     return conditional(cond(w), 2*w, 3*w)
+        def df(w, v): return (conditional(cond(w), 1, 0) * 2*v +
+                              conditional(cond(w), 0, 1) * 3*v)
+        self._test(f, df)
+
     # --- Tensor algebra basics
 
     def testIndexSum(self):
@@ -222,13 +240,14 @@ class DerivativeTestCase(UflTestCase):
 
     def assertEqualBySampling(self, actual, expected):
         ad = (actual*dx).compute_form_data()
-        a = ad.preprocessed_form.cell_integrals()[0].integrand()
+        a = ad.preprocessed_form.integrals(Measure.CELL)[0].integrand()
         bd = (expected*dx).compute_form_data()
-        b = bd.preprocessed_form.cell_integrals()[0].integrand()
+        b = bd.preprocessed_form.integrals(Measure.CELL)[0].integrand()
 
-        self.assertEqual(ad.coefficients, bd.coefficients)
+        self.assertEqual([ad.function_replace_map[ac] for ac in ad.original_coefficients],
+                         [bd.function_replace_map[bc] for bc in bd.original_coefficients])
 
-        n = len(ad.coefficients)
+        n = ad.num_coefficients
         def make_value(c):
             z = 0.3 if isinstance(c, Coefficient) else 0.7
             if c.shape() == ():
@@ -236,9 +255,9 @@ class DerivativeTestCase(UflTestCase):
             elif len(c.shape()) == 1:
                 return tuple((z * (j + 0.1 + 0.9 * c.count() / n) for j in range(c.shape()[0])))
             else:
-                raise NotImplemented
-        amapping = dict((c, make_value(c)) for c in chain(ad.coefficients, ad.arguments))
-        bmapping = dict((c, make_value(c)) for c in chain(bd.coefficients, bd.arguments))
+                raise NotImplementedError("Tensor valued expressions not supported here.")
+        amapping = dict((c, make_value(c)) for c in chain(ad.original_coefficients, ad.original_arguments))
+        bmapping = dict((c, make_value(c)) for c in chain(bd.original_coefficients, bd.original_arguments))
 
         acell = actual.cell()
         bcell = expected.cell()
@@ -359,7 +378,10 @@ class DerivativeTestCase(UflTestCase):
 
         for i in range(cell.d):
             for j in range(cell.d):
-                a = expand_indices((Lvu[i,j]*dx).compute_form_data().preprocessed_form)
+                form = Lvu[i,j]*dx
+                fd = form.compute_form_data()
+                pf = fd.preprocessed_form
+                a = expand_indices(pf)
                 #print (i,j), str(a)
 
         k = Index()
@@ -391,7 +413,8 @@ class DerivativeTestCase(UflTestCase):
         J = derivative(F, u, du, cd)
         fd = J.compute_form_data()
         actual = fd.preprocessed_form.integrals()[0].integrand()
-        self.assertEqual(actual, expected)
+        self.assertEqual((actual*dx).deprecated_signature(), (expected*dx).deprecated_signature())
+        self.assertEqual(replace(actual, fd.function_replace_map), expected)
 
     def test_vector_coefficient_derivatives(self):
         V = VectorElement("Lagrange", triangle, 1)
@@ -415,7 +438,8 @@ class DerivativeTestCase(UflTestCase):
         J = derivative(F, u, du, cd)
         fd = J.compute_form_data()
         actual = fd.preprocessed_form.integrals()[0].integrand()
-        self.assertEqual(actual, expected)
+        self.assertEqual((actual*dx).deprecated_signature(), (expected*dx).deprecated_signature())
+        #self.assertEqual(replace(actual, fd.function_replace_map), expected)
 
     def test_vector_coefficient_derivatives_of_product(self):
         V = VectorElement("Lagrange", triangle, 1)
@@ -463,7 +487,7 @@ class DerivativeTestCase(UflTestCase):
         # Sampling the expressions instead of comparing representations.
         x = (0, 0)
         funcs = {dv: (13, 14), f: (1,2), g: (3,4), df: ((5,6),(7,8)), dg: ((9,10),(11,12))}
-        self.assertEqual(actual(x, funcs), expected(x, funcs))
+        self.assertEqual(replace(actual,fd.function_replace_map)(x, funcs), expected(x, funcs))
 
         # TODO: Add tests covering more cases, in particular mixed stuff
 
@@ -499,9 +523,9 @@ class DerivativeTestCase(UflTestCase):
         F = form_data_F.preprocessed_form
         J = form_data_J.preprocessed_form
 
-        f_expression = strip_variables(f.cell_integrals()[0].integrand())
-        F_expression = strip_variables(F.cell_integrals()[0].integrand())
-        J_expression = strip_variables(J.cell_integrals()[0].integrand())
+        f_expression = strip_variables(f.integrals(Measure.CELL)[0].integrand())
+        F_expression = strip_variables(F.integrals(Measure.CELL)[0].integrand())
+        J_expression = strip_variables(J.integrals(Measure.CELL)[0].integrand())
 
         #classes = set(c.__class__ for c in post_traversal(f_expression))
 
@@ -530,19 +554,19 @@ class DerivativeTestCase(UflTestCase):
             assert derivatives == (0,)
             return dw
 
-        w, b, K = form_data_f.coefficients
+        w, b, K = form_data_f.original_coefficients
         mapping = { K: Kv, b: bv, w: Nw }
         fv2 = f_expression((0,), mapping)
         self.assertAlmostEqual(fv, fv2)
 
-        w, b, K = form_data_F.coefficients
-        v, = form_data_F.arguments
+        w, b, K = form_data_F.original_coefficients
+        v, = form_data_F.original_arguments
         mapping = { K: Kv, b: bv, v: Nv, w: Nw }
         Fv2 = F_expression((0,), mapping)
         self.assertAlmostEqual(Fv, Fv2)
 
-        w, b, K = form_data_J.coefficients
-        v, u = form_data_J.arguments
+        w, b, K = form_data_J.original_coefficients
+        v, u = form_data_J.original_arguments
         mapping = { K: Kv, b: bv, v: Nv, u: Nu, w: Nw }
         Jv2 = J_expression((0,), mapping)
         self.assertAlmostEqual(Jv, Jv2)
@@ -563,6 +587,30 @@ class DerivativeTestCase(UflTestCase):
         J2 = derivative(F, w, u)
         # TODO: assert something
 
+    # --- Interaction with replace
+
+    def test_derivative_replace_works_together(self):
+        cell = triangle
+        V = FiniteElement("CG", cell, 1)
+
+        v = TestFunction(V)
+        u = TrialFunction(V)
+        f = Coefficient(V)
+        g = Coefficient(V)
+
+        M = cos(f)*sin(g)
+        F = derivative(M, f, v)
+        J = derivative(F, f, u)
+        JR = replace(J, { f: g })
+
+        F2 = -sin(f)*v*sin(g)
+        J2 = -cos(f)*u*v*sin(g)
+        JR2 = -cos(g)*u*v*sin(g)
+
+        self.assertEqualBySampling(F, F2)
+        self.assertEqualBySampling(J, J2)
+        self.assertEqualBySampling(JR, JR2)
+
     # --- Scratch space
 
     def test_foobar(self):
