@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 __authors__ = "Martin Sandve Alnes"
-__date__ = "2009-03-19 -- 2009-03-24"
+__date__ = "2009-03-19 -- 2012-03-20"
 
 # Modified by Anders Logg, 2008
 # Modified by Garth N. Wells, 2009
@@ -14,7 +14,8 @@ from ufl import *
 from ufl.algorithms import * 
 from ufl.classes import Sum, Product
 
-# TODO: add more tests, covering all utility algorithms
+# TODO: Test expand_indices2 throuroughly for correctness, then efficiency:
+#expand_indices, expand_indices2 = expand_indices2, expand_indices
 
 class ExpandIndicesTestCase(UflTestCase):
 
@@ -32,60 +33,91 @@ class ExpandIndicesTestCase(UflTestCase):
         #       their unique constant values are used for validation.
         
         def SF(x, derivatives=()): 
-            if derivatives == ():
-                return 3
-            elif derivatives == (0,):
+            # first order derivatives
+            if derivatives == (0,):
                 return 0.30
             elif derivatives == (1,):
                 return 0.31
-            return 0
-        
+            # second order derivatives
+            elif derivatives == (0,0):
+                return 0
+            elif derivatives in ((1,0), (0,1)):
+                return 0
+            elif derivatives == (1,1):
+                return 0
+            # function value
+            assert derivatives == ()
+            return 3
+
         def SF2(x, derivatives=()):
-            if derivatives == ():
-                return 3
-            elif derivatives == (0,):
+            # first order derivatives
+            if derivatives == (0,):
                 return 0.30
             elif derivatives == (1,):
                 return 0.31
+            # second order derivatives
             elif derivatives == (0,0):
                 return 3.300
             elif derivatives in ((1,0), (0,1)):
                 return 3.310
             elif derivatives == (1,1):
                 return 3.311
-            return 0
+            # function value
+            assert derivatives == ()
+            return 3
         
         def VF(x, derivatives=()):
-            if derivatives == ():
-                return (5, 7)
-            elif derivatives == (0,):
+            # first order derivatives
+            if derivatives == (0,):
                 return (0.50, 0.70)
             elif derivatives == (1,):
                 return (0.51, 0.71)
-            return (0, 0)
+            # second order derivatives
+            elif derivatives == (0,0):
+                return (0.20, 0.21)
+            elif derivatives in ((1,0), (0,1)):
+                return (0.30, 0.31)
+            elif derivatives == (1,1):
+                return (0.40, 0.41)
+            # function value
+            assert derivatives == ()
+            return (5, 7)
         
         def TF(x, derivatives=()):
-            if derivatives == ():
-                return ((11, 13), (17, 19))
-            elif derivatives == (0,):
+            # first order derivatives
+            if derivatives == (0,):
                 return ((1.10, 1.30), (1.70, 1.90))
             elif derivatives == (1,):
                 return ((1.11, 1.31), (1.71, 1.91))
-            return ((0, 0), (0, 0))
+            # second order derivatives
+            elif derivatives == (0,0):
+                return ((10.00, 10.01), (10.10, 10.11))
+            elif derivatives in ((1,0), (0,1)):
+                return ((12.00, 12.01), (12.10, 12.11))
+            elif derivatives == (1,1):
+                return ((11.00, 11.01), (11.10, 11.11))
+            # function value
+            assert derivatives == ()
+            return ((11, 13), (17, 19))
         
         self.x = (1.23, 3.14)
         self.mapping = { self.sf: SF, self.sf2: SF2, self.vf: VF, self.tf: TF }
         
     def compare(self, f, value):
+        debug = 0
+        if debug: print 'f', f
         g = expand_derivatives(f)
+        if debug: print 'g', g
         gv = g(self.x, self.mapping)
         self.assertAlmostEqual(gv, value)
 
         g = expand_indices(g)
+        if debug: print 'g', g
         gv = g(self.x, self.mapping)
         self.assertAlmostEqual(gv, value)
 
         g = renumber_indices(g)
+        if debug: print 'g', g
         gv = g(self.x, self.mapping)
         self.assertAlmostEqual(gv, value)
 
@@ -237,6 +269,23 @@ class ExpandIndicesTestCase(UflTestCase):
         a = div(grad(sf2))
         compare(a, 3.300 + 3.311)
 
+        if 0:
+            Dvf = grad(vf)
+            Lvf = div(Dvf)
+            Lvf2 = dot(Lvf,Lvf)
+            pp = (Lvf2*dx).compute_form_data().preprocessed_form.integrals()[0].integrand()
+            print 'vf', vf.shape(), str(vf)
+            print 'Dvf', Dvf.shape(), str(Dvf)
+            print 'Lvf', Lvf.shape(), str(Lvf)
+            print 'Lvf2', Lvf2.shape(), str(Lvf2)
+            print 'pp', pp.shape(), str(pp)
+
+        a = div(grad(vf))
+        compare(dot(a,a), (0.20+0.40)**2 + (0.21+0.41)**2)
+
+        a = div(grad(tf))
+        compare(inner(a,a), (10.00+11.00)**2 + (10.01+11.01)**2 + (10.10+11.10)**2 + (10.11+11.11)**2)
+
     def test_expand_indices_nabla_div_grad(self):
         sf = self.sf
         sf2 = self.sf2
@@ -250,5 +299,34 @@ class ExpandIndicesTestCase(UflTestCase):
         a = nabla_div(nabla_grad(sf2))
         compare(a, 3.300 + 3.311)
 
+        a = nabla_div(nabla_grad(vf))
+        compare(dot(a,a), (0.20+0.40)**2 + (0.21+0.41)**2)
+
+        a = nabla_div(nabla_grad(tf))
+        compare(inner(a,a), (10.00+11.00)**2 + (10.01+11.01)**2 + (10.10+11.10)**2 + (10.11+11.11)**2)
+
+    def xtest_expand_indices_list_tensor_problem(self):
+        print
+        print '='*40
+        # TODO: This is the case marked in the expand_indices2 implementation
+        #as not working. Fix and then try expand_indices2 on other tests!
+        V = VectorElement("CG", triangle, 1)
+        w = Coefficient(V)
+        v = as_vector([w[0], 0])
+        a = v[i]*w[i]
+        # TODO: Compare
+        print type(a), str(a)
+        A, comp = a.operands()
+        print type(A), str(A)
+        print type(comp), str(comp)
+
+        ei1 = expand_indices(a)
+        ei2 = expand_indices2(a)
+        print str(ei1)
+        print str(ei2)
+        print '='*40
+        print
+
 if __name__ == "__main__":
     main()
+
