@@ -4,7 +4,7 @@ discretization. More precisely, it defines a fixed interface for choosing
 finite element spaces and defining expressions for weak forms in a
 notation close to mathematical notation.
 
-This python module contains the language as well as algorithms to work
+This Python module contains the language as well as algorithms to work
 with it.
 
 * To import the language, type::
@@ -28,9 +28,13 @@ For more details on the language, see
 
   http://www.fenicsproject.org
 
+and
+
+  http://arxiv.org/abs/1211.4047
+
 The development version can be found in the repository at
 
-  https://launchpad.net/ufl/
+  https://www.bitbucket.org/fenics-project/ufl
 
 A very brief overview of the language contents follows:
 
@@ -72,7 +76,7 @@ A very brief overview of the language contents follows:
 
     SpatialCoordinate, FacetNormal,
     CellVolume, Circumradius, CellSurfaceArea,
-    FacetArea, FacetDiameter,
+    FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength, FacetDiameter,
     LocalCoordinate, GeometryJacobi,
     GeometryJacobiDeterminant, InverseGeometryJacobi
 
@@ -113,12 +117,12 @@ A very brief overview of the language contents follows:
     abs, sign, sqrt,
     exp, ln, erf,
     cos, sin, tan,
-    acos, asin, atan,
+    acos, asin, atan, atan_2,
     cosh, sinh, tanh,
     bessel_J, bessel_Y, bessel_I, bessel_K
 
 * Discontinuous Galerkin operators:
-    jump, avg, v('+'), v('-')
+    jump, avg, v('+'), v('-'), cell_avg, facet_avg
 
 * Conditional operators::
 
@@ -157,10 +161,11 @@ A very brief overview of the language contents follows:
 #
 # Modified by Kristian B. Oelgaard, 2009, 2011
 # Modified by Anders Logg, 2009.
+# Modified by Johannes Ring, 2014.
 #
-# Last changed: 2013-03-15
+# Last changed: 2014-01-07
 
-__version__ = "1.2.0"
+__version__ = "1.3.0"
 
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
@@ -181,7 +186,7 @@ from ufl.log import get_handler, get_logger, set_handler, set_level, add_logfile
 from ufl.geometry import (Cell, ProductCell,
      SpatialCoordinate, FacetNormal,
      CellVolume, Circumradius, CellSurfaceArea,
-     FacetArea, FacetDiameter,
+     FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength, FacetDiameter,
      LocalCoordinate, GeometryJacobi,
      GeometryJacobiDeterminant, InverseGeometryJacobi)
 
@@ -229,14 +234,14 @@ from ufl.operators import rank, shape, \
                        dev, skew, sym, \
                        sqrt, exp, ln, erf, \
                        cos, sin, tan, \
-                       acos, asin, atan, \
+                       acos, asin, atan, atan_2, \
                        cosh, sinh, tanh, \
                        bessel_J, bessel_Y, bessel_I, bessel_K, \
                        eq, ne, le, ge, lt, gt, And, Or, Not, \
                        conditional, sign, Max, Min, \
                        variable, diff, \
                        Dx,  grad, div, curl, rot, nabla_grad, nabla_div, Dn, exterior_derivative, \
-                       jump, avg, \
+                       jump, avg, cell_avg, facet_avg, \
                        elem_mult, elem_div, elem_pow, elem_op
 
 # Form class
@@ -266,7 +271,7 @@ __all__ = [
     'Cell', 'ProductCell',
     'SpatialCoordinate', 'FacetNormal',
     'CellVolume', 'Circumradius', 'CellSurfaceArea',
-    'FacetArea', 'FacetDiameter',
+    'FacetArea', 'MinFacetEdgeLength', 'MaxFacetEdgeLength', 'FacetDiameter',
     'LocalCoordinate', 'GeometryJacobi',
     'GeometryJacobiDeterminant', 'InverseGeometryJacobi',
     'Domain', 'Region',
@@ -289,14 +294,14 @@ __all__ = [
     'transpose', 'tr', 'diag', 'diag_vector', 'dev', 'skew', 'sym',
     'sqrt', 'exp', 'ln', 'erf',
     'cos', 'sin', 'tan',
-    'acos', 'asin', 'atan',
+    'acos', 'asin', 'atan', 'atan_2',
     'cosh', 'sinh', 'tanh',
     'bessel_J', 'bessel_Y', 'bessel_I', 'bessel_K',
     'eq', 'ne', 'le', 'ge', 'lt', 'gt', 'And', 'Or', 'Not',
     'conditional', 'sign', 'Max', 'Min',
     'variable', 'diff',
     'Dx', 'grad', 'div', 'curl', 'rot', 'nabla_grad', 'nabla_div', 'Dn', 'exterior_derivative',
-    'jump', 'avg',
+    'jump', 'avg', 'cell_avg', 'facet_avg',
     'elem_mult', 'elem_div', 'elem_pow', 'elem_op',
     'Form',
     'Integral', 'Measure', 'register_domain_type', 'ProductMeasure',
