@@ -59,8 +59,7 @@ from ufl.algorithms.replace import Replacer, replace
 from ufl.algorithms.expand_compounds import CompoundExpander, expand_compounds, \
                                             CompoundExpanderPreDiff, expand_compounds_prediff, \
                                             CompoundExpanderPostDiff, expand_compounds_postdiff
-from ufl.algorithms.estimate_degrees import MaxDegreeEstimator, estimate_max_polynomial_degree, \
-                                            SumDegreeEstimator, estimate_total_polynomial_degree
+from ufl.algorithms.estimate_degrees import SumDegreeEstimator, estimate_total_polynomial_degree
 from ufl.algorithms.argument_dependencies import ArgumentDependencyExtracter, extract_argument_dependencies, NotMultiLinearException
 from ufl.algorithms.deprecated import TreeFlattener, flatten, \
                                       DuplicationMarker, mark_duplications, \
