@@ -1,6 +1,6 @@
 """Functions to check the validity of forms."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,10 +21,9 @@
 # Modified by Mehdi Nikbakht, 2010.
 #
 # First added:  2008-03-14
-# Last changed: 2011-06-08
+# Last changed: 2012-04-12
 
 from ufl.log import warning, error
-from ufl.assertions import ufl_assert
 
 # UFL classes
 from ufl.form import Form
@@ -34,15 +33,13 @@ from ufl.constantvalue import is_true_ufl_scalar
 from ufl.integral import Measure
 
 # UFL algorithms
-from ufl.algorithms.traversal import iter_expressions, traverse_terminals, fast_pre_traversal
-from ufl.algorithms.analysis import extract_elements
-from ufl.algorithms.predicates import is_multilinear
+from ufl.algorithms.traversal import iter_expressions, traverse_terminals
 from ufl.algorithms.propagate_restrictions import check_restrictions
 
 def validate_form(form): # TODO: Can we make this return a list of errors instead of raising exception?
     """Performs all implemented validations on a form. Raises exception if something fails."""
-
     errors = []
+    warnings = []
 
     if not isinstance(form, Form):
         msg = "Validation failed, not a Form:\n%s" % repr(form)
@@ -55,13 +52,21 @@ def validate_form(form): # TODO: Can we make this return a list of errors instea
     #if not is_multilinear(form):
     #    errors.append("Form is not multilinear in arguments.")
 
+    # FIXME DOMAIN: Add check for consistency between domains somehow
+    domains = set(t.domain()
+                  for e in iter_expressions(form)
+                  for t in traverse_terminals(e)) - set((None,))
+    if not domains:
+        errors.append("Missing domain definition in form.")
+
+    top_domains = set(dom.top_domain() for dom in domains if dom is not None)
+    if not top_domains:
+        errors.append("Missing domain definition in form.")
+    elif len(top_domains) > 1:
+        warnings.append("Multiple top domain definitions in form: %s" % str(top_domains))
+
     # Check that cell is the same everywhere
-    cells = set()
-    for e in iter_expressions(form):
-        cells.update(t.cell() for t in traverse_terminals(e) \
-                     if not (t.cell() is None or t.cell().domain() is None))
-    if None in cells:
-        cells.remove(None)
+    cells = set(dom.cell() for dom in top_domains) - set((None,))
     if not cells:
         errors.append("Missing cell definition in form.")
     elif len(cells) > 1:
