@@ -1,4 +1,4 @@
-# Copyright (C) 2011 Marie E. Rognes
+# Copyright (C) 2012 Marie E. Rognes
 #
 # This file is part of UFL.
 #
@@ -18,6 +18,7 @@
 # First added:  2011-01-17
 # Last changed: 2011-06-02
 
+from ufl.assertions import ufl_assert
 from ufl.finiteelement import FiniteElement, MixedElement
 
 def change_regularity(element, family):
@@ -33,9 +34,10 @@ def change_regularity(element, family):
                              for i in range(n)])
     shape = element.value_shape()
     if not shape:
-        return FiniteElement(family, element.cell(), element.degree())
+        return FiniteElement(family, element.domain(), element.degree())
 
-    return MixedElement([FiniteElement(family, element.cell(), element.degree())
+    ufl_assert(len(shape) == 1, "TODO: Update this code to handle tensor elements.")
+    return MixedElement([FiniteElement(family, element.domain(i), element.degree(i))
                                for i in range(shape[0])])
 
 def tear(V):
@@ -45,6 +47,7 @@ def tear(V):
 
 def increase_order(element):
     "Return element of same family, but a polynomial degree higher."
+    ufl_assert(len(element.value_shape()) <= 1, "TODO: Update this code to handle tensor elements.")
 
     n = element.num_sub_elements()
     if n > 0:
@@ -54,4 +57,4 @@ def increase_order(element):
     if element.family() == "Real":
         return element
 
-    return FiniteElement(element.family(), element.cell(), element.degree()+1)
+    return FiniteElement(element.family(), element.domain(), element.degree()+1)
