@@ -23,7 +23,7 @@
 # Modified by Jan Blechta, 2012.
 #
 # First added:  2008-08-19
-# Last changed: 2012-10-03
+# Last changed: 2013-08-12
 
 from itertools import izip
 from math import pi
@@ -46,15 +46,16 @@ from ufl.tensors import ListTensor, ComponentTensor, as_tensor, as_scalar, unit_
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.tensoralgebra import Transposed, Outer, Inner, Dot, Cross, Trace, \
     Determinant, Inverse, Deviatoric, Cofactor
-from ufl.mathfunctions import MathFunction, Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
+from ufl.mathfunctions import MathFunction, Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan, Atan2, Erf, BesselJ, BesselY, BesselI, BesselK
 from ufl.restriction import Restricted, PositiveRestricted, NegativeRestricted
 from ufl.differentiation import Derivative, CoefficientDerivative,\
     VariableDerivative, Grad
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 
 from ufl.operators import dot, inner, outer, lt, eq, conditional, sign, \
-    sqrt, exp, ln, cos, sin, tan, cosh, sinh, tanh, acos, asin, atan, \
-    erf, bessel_J, bessel_Y, bessel_I, bessel_K
+    sqrt, exp, ln, cos, sin, tan, cosh, sinh, tanh, acos, asin, atan, atan_2, \
+    erf, bessel_J, bessel_Y, bessel_I, bessel_K, \
+    cell_avg, facet_avg
 from ufl.algorithms.transformer import Transformer
 
 
@@ -483,6 +484,13 @@ class ForwardAD(Transformer):
         op = fp/(1.0 + f**2)
         return (o, op)
 
+    def atan_2(self, o, a, b):
+        f, fp = a
+        g, gp = b
+        o = self.reuse_if_possible(o, f, g)
+        op = (g*fp-f*gp)/(f**2+g**2)
+        return (o, op)
+
     def erf(self, o, a):
         f, fp = a
         o = self.reuse_if_possible(o, f)
@@ -548,6 +556,24 @@ class ForwardAD(Transformer):
         else:
             return (o, fp(o._side)) # (f+-)' == (f')+-
 
+    def cell_avg(self, o, a):
+        # Cell average of a single function and differentiation commutes.
+        f, fp = a
+        o = self.reuse_if_possible(o, f)
+        if isinstance(fp, ConstantValue):
+            return (o, fp) # TODO: Necessary? Can't CellAvg simplify directly instead?
+        else:
+            return (o, cell_avg(fp))
+
+    def facet_avg(self, o, a):
+        # Facet average of a single function and differentiation commutes.
+        f, fp = a
+        o = self.reuse_if_possible(o, f)
+        if isinstance(fp, ConstantValue):
+            return (o, fp) # TODO: Necessary? Can't FacetAvg simplify directly instead?
+        else:
+            return (o, facet_avg(fp))
+
     # --- Conditionals
 
     def binary_condition(self, o, l, r):
@@ -908,20 +934,17 @@ class CoefficientAD(ForwardAD):
         self._variable_cache[l] = c
         return c
 
-def compute_grad_forward_ad(expr, dim):
-    f, = expr.operands()
+def compute_grad_forward_ad(f, dim):
     alg = GradAD(dim)
     e, ediff = alg.visit(f)
     return ediff
 
-def compute_variable_forward_ad(expr, dim):
-    f, v = expr.operands()
+def compute_variable_forward_ad(f, v, dim):
     alg = VariableAD(dim, v)
     e, ediff = alg.visit(f)
     return ediff
 
-def compute_coefficient_forward_ad(expr, dim):
-    f, w, v, cd = expr.operands()
+def compute_coefficient_forward_ad(f, w, v, cd, dim):
     alg = CoefficientAD(dim, w, v, cd)
     e, ediff = alg.visit(f)
     return ediff
@@ -930,36 +953,35 @@ def apply_nested_forward_ad(expr, dim):
     if isinstance(expr, Terminal):
         # A terminal needs no differentiation applied
         return expr
-    else:
-        # Store the original expression here, because
-        # reconstruct below may simplify and make expr
-        # get the type of a child, which may be a derivative,
-        # even if preexpr here is not a derivative.
-        preexpr = expr
-
+    elif not isinstance(expr, Derivative):
         # Apply AD recursively to children
         preops = expr.operands()
         postops = tuple(apply_nested_forward_ad(o, dim) for o in preops)
-
+        # Reconstruct if necessary
         need_reconstruct = not (preops == postops) # FIXME: Is this efficient? O(n)?
         if need_reconstruct:
             expr = expr.reconstruct(*postops)
-
-        # Preliminary result
-        result = expr
-
-        # Check if this node is a derivative, if so apply AD to it
-        if isinstance(preexpr, Derivative):
-            if isinstance(expr, Grad):
-                result = compute_grad_forward_ad(expr, dim)
-            elif isinstance(preexpr, VariableDerivative):
-                result = compute_variable_forward_ad(expr, dim)
-            elif isinstance(preexpr, CoefficientDerivative):
-                result = compute_coefficient_forward_ad(expr, dim)
-            else:
-                error("This shouldn't happen: expr is %s" % repr(expr))
-        # Return the expanded result
-        return result
+        return expr
+    elif isinstance(expr, Grad):
+        # Apply AD recursively to children
+        f, = expr.operands()
+        f = apply_nested_forward_ad(f, dim)
+        # Apply Grad-specialized AD to expanded child
+        return compute_grad_forward_ad(f, dim)
+    elif isinstance(expr, VariableDerivative):
+        # Apply AD recursively to children
+        f, v = expr.operands()
+        f = apply_nested_forward_ad(f, dim)
+        # Apply Variable-specialized AD to expanded child
+        return compute_variable_forward_ad(f, v, dim)
+    elif isinstance(expr, CoefficientDerivative):
+        # Apply AD recursively to children
+        f, w, v, cd = expr.operands()
+        f = apply_nested_forward_ad(f, dim)
+        # Apply Coefficient-specialized AD to expanded child
+        return compute_coefficient_forward_ad(f, w, v, cd, dim)
+    else:
+        error("Unknown type.")
 
 # TODO: We could expand only the compound objects that have no rule
 #       before differentiating, to allow the AD to work on a coarser graph
