@@ -1,6 +1,6 @@
 "Algorithms related to restrictions."
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,17 +18,18 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-05-14
-# Last changed: 2011-11-17
+# Last changed: 2012-04-12
 
 from ufl.expr import Expr
+from ufl.classes import Measure
 from ufl.assertions import ufl_assert
-from ufl.algorithms.transformations import Transformer, ReuseTransformer
+from ufl.algorithms.transformer import Transformer, ReuseTransformer, apply_transformer
 
 class RestrictionPropagator(ReuseTransformer):
     def __init__(self):
         ReuseTransformer.__init__(self)
         self.current_restriction = None
-    
+
     def restricted(self, o):
         ufl_assert(self.current_restriction is None,
             "Not expecting twice restricted expression.")
@@ -62,7 +63,7 @@ class RestrictionPropagator(ReuseTransformer):
         #    return o
         return o(self.current_restriction)
 
-    # facet_area is the same from both sides of a facet
+    # facet_area, facet_diameter, max_facet_edge_length are all the same from both sides of a facet
 
     def form_argument(self, o):
         ufl_assert(self.current_restriction is not None, "Form argument must be restricted.")
@@ -106,10 +107,9 @@ class RestrictionChecker(Transformer):
             ufl_assert(self.current_restriction is None, "Restrictions are only allowed for interior facet integrals.")
 
 def propagate_restrictions(expression):
-    ufl_assert(isinstance(expression, Expr), "Expecting Expr instance.")
-    return RestrictionPropagator().visit(expression)
+    "Propagate restriction nodes to wrap terminal objects directly."
+    return apply_transformer(expression, RestrictionPropagator(), domain_type=Measure.INTERIOR_FACET)
 
 def check_restrictions(expression, require_restriction):
     ufl_assert(isinstance(expression, Expr), "Expecting Expr instance.")
     return RestrictionChecker(require_restriction).visit(expression)
-
