@@ -1,6 +1,6 @@
 "Algorithms for renumbering of counted objects, currently variables and indices."
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2013 Martin Sandve Alnes and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -18,16 +18,17 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-02-22
-# Last changed: 2011-06-02
+# Last changed: 2012-04-12
 
-from ufl.common import Counted, Stack, StackDict
+from itertools import izip
+from ufl.common import Stack, StackDict
 from ufl.log import error
 from ufl.expr import Expr
-from ufl.indexing import Index, FixedIndex, MultiIndex, Indexed
+from ufl.indexing import Index, FixedIndex, MultiIndex
+from ufl.indexed import Indexed
 from ufl.indexsum import IndexSum
-from ufl.tensors import ComponentTensor, ListTensor
 from ufl.variable import Label, Variable
-from ufl.algorithms.transformations import ReuseTransformer, apply_transformer
+from ufl.algorithms.transformer import ReuseTransformer, apply_transformer
 
 class VariableRenumberingTransformer(ReuseTransformer):
     def __init__(self):
@@ -59,7 +60,7 @@ class IndexRenumberingTransformer(VariableRenumberingTransformer):
     def multi_index(self, o):
         new_indices = tuple(map(self.index, o._indices))
         idims = o.index_dimensions()
-        new_idims = dict((b, idims[a]) for (a,b) in zip(o._indices, new_indices) if isinstance(a, Index))
+        new_idims = dict((b, idims[a]) for (a,b) in izip(o._indices, new_indices) if isinstance(a, Index))
         return MultiIndex(new_indices, new_idims)
 
     def index(self, o):
@@ -119,7 +120,7 @@ class IndexRenumberingTransformer2(VariableRenumberingTransformer):
         return tuple(ni)
 
     def define_indices(self, ii, values):
-        for i, v in zip(ii, values):
+        for i, v in izip(ii, values):
             #print "::: Defining index", repr(i), "= ", repr(v)
             if v is None:
                 if self.index_map.get(i) is None:
@@ -163,7 +164,7 @@ class IndexRenumberingTransformer2(VariableRenumberingTransformer):
     def multi_index(self, o):
         new_indices = tuple(map(self.index, o._indices))
         idims = o.index_dimensions()
-        new_idims = dict((b, idims[a]) for (a,b) in zip(o._indices, new_indices) if isinstance(a, Index))
+        new_idims = dict((b, idims[a]) for (a,b) in izip(o._indices, new_indices) if isinstance(a, Index))
         return MultiIndex(new_indices, new_idims)
 
     def index_annotated(self, o):
