@@ -40,8 +40,8 @@ from ufl.tensors import ListTensor, ComponentTensor
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.indexsum import IndexSum
 from ufl.tensoralgebra import Transposed, Outer, Inner, Dot, Cross, Trace, Determinant, Inverse, Deviatoric, Cofactor
-from ufl.mathfunctions import Sqrt, Exp, Ln, Cos, Sin, Tan, Cosh, Sinh, Tanh, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
-from ufl.restriction import PositiveRestricted, NegativeRestricted
+from ufl.mathfunctions import Sqrt, Exp, Ln, Cos, Sin, Tan, Cosh, Sinh, Tanh, Acos, Asin, Atan, Atan2, Erf, BesselJ, BesselY, BesselI, BesselK
+from ufl.restriction import PositiveRestricted, NegativeRestricted, CellAvg, FacetAvg
 from ufl.differentiation import VariableDerivative, Grad, Div, Curl, NablaGrad, NablaDiv
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 from ufl.form import Form
@@ -73,6 +73,7 @@ def build_precedence_map():
 
     # TODO: What to do with these?
     precedence_list.append((ListTensor, ComponentTensor))
+    precedence_list.append((CellAvg, ))
     precedence_list.append((NegativeRestricted, PositiveRestricted))
     precedence_list.append((Conditional,))
     precedence_list.append((LE, GT, GE, NE, EQ, LT))
@@ -81,7 +82,7 @@ def build_precedence_map():
                             Determinant, Trace, Cofactor, Inverse, Deviatoric))
     precedence_list.append((Product, Division, Cross, Dot, Outer, Inner))
     precedence_list.append((Indexed, Transposed, Power))
-    precedence_list.append((Abs, Cos, Cosh, Exp, Ln, Sin, Sinh, Sqrt, Tan, Tanh, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK))
+    precedence_list.append((Abs, Cos, Cosh, Exp, Ln, Sin, Sinh, Sqrt, Tan, Tanh, Acos, Asin, Atan, Atan2, Erf, BesselJ, BesselY, BesselI, BesselK))
     precedence_list.append((Variable,))
     precedence_list.append(terminal_classes)
 
@@ -249,6 +250,9 @@ class Expression2LatexHandler(Transformer):
     def atan(self, o, f):
         return r"\arctan{%s}" % par(f)
 
+    def atan2(self, o, f1, f2):
+        return r"\arctan_2{%s,%s}" % (par(f1), par(f2))
+
     def erf(self, o, f):
         return r"\erf{%s}" % par(f)
 
@@ -325,6 +329,9 @@ class Expression2LatexHandler(Transformer):
     def negative_restricted(self, o, f):
         return "{%s}^-" % par(f)
 
+    def cell_avg(self, o, f):
+        return "{%s}_K" % par(f)
+
     def eq(self, o, a, b):
         return "(%s = %s)" % (a, b)
 
@@ -399,27 +406,27 @@ def form2latex(form, formdata):
 
     # Define elements
     lines = []
-    for i, f in enumerate(formdata.arguments):
+    for i, f in enumerate(formdata.original_arguments):
         lines.append(r"\mathcal{P}_{%d} = \{%s\} " % (i, element2latex(f.element())))
-    for i, f in enumerate(formdata.coefficients):
+    for i, f in enumerate(formdata.original_coefficients):
         lines.append(r"\mathcal{Q}_{%d} = \{%s\} " % (i, element2latex(f.element())))
     if lines:
         sections.append(("Finite elements", align(lines)))
 
     # Define function spaces
     lines = []
-    for i, f in enumerate(formdata.arguments):
+    for i, f in enumerate(formdata.original_arguments):
         lines.append("V_h^{%d} = \\{v : v \\vert_K \\in \\mathcal{P}_{%d}(K) \\quad \\forall K \\in \\mathcal{T}\\} " % (i, i))
-    for i, f in enumerate(formdata.coefficients):
+    for i, f in enumerate(formdata.original_coefficients):
         lines.append("W_h^{%d} = \\{v : v \\vert_K \\in \\mathcal{Q}_{%d}(K) \\quad \\forall K \\in \\mathcal{T}\\} " % (i, i))
     if lines:
         sections.append(("Function spaces", align(lines)))
 
     # Define arguments and coefficients
     lines = []
-    for i, f in enumerate(formdata.arguments):
+    for i, f in enumerate(formdata.original_arguments):
         lines.append("%s = %s \\in V_h^{%d} " % (argument_names[i], bfname(i), i))
-    for i, f in enumerate(formdata.coefficients):
+    for i, f in enumerate(formdata.original_coefficients):
         lines.append("%s = %s \\in W_h^{%d} " % (coefficient_names[i], cfname(i), i))
     if lines:
         sections.append(("Form arguments", align(lines)))
