@@ -1,6 +1,6 @@
 "Differential operators."
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -24,17 +24,15 @@
 
 from ufl.log import warning, error
 from ufl.assertions import ufl_assert
-from ufl.common import subdict, mergedicts
-from ufl.expr import Expr, Operator
-from ufl.terminal import Terminal, Tuple, Data
-from ufl.constantvalue import ConstantValue, Zero, ScalarValue, Identity, is_true_ufl_scalar
-from ufl.indexing import Index, FixedIndex, Indexed, as_multi_index, MultiIndex
+from ufl.common import subdict, mergedicts, EmptyDict
+from ufl.expr import Expr
+from ufl.terminal import Terminal, Data
+from ufl.operatorbase import Operator, Tuple
+from ufl.constantvalue import Zero
+from ufl.indexing import Index, FixedIndex, MultiIndex, as_multi_index
+from ufl.indexed import Indexed
 from ufl.indexutils import unique_indices
-from ufl.geometry import FacetNormal, CellVolume, Circumradius
 from ufl.variable import Variable
-from ufl.tensors import as_tensor, ComponentTensor, ListTensor
-from ufl.argument import Argument
-from ufl.coefficient import Coefficient, ConstantBase
 from ufl.precedence import parstr
 
 #--- Basic differentiation objects ---
@@ -52,22 +50,15 @@ class CoefficientDerivative(Derivative):
                  "_coefficient_derivatives")
 
     def __new__(cls, integrand, coefficients, arguments, coefficient_derivatives):
-        ufl_assert(is_true_ufl_scalar(integrand),
-            "Expecting true UFL scalar expression.")
-
         ufl_assert(isinstance(coefficients, Tuple),
             "Expecting Tuple instance with Coefficients.")
-        #and all(isinstance(f, (Coefficient, Indexed)) for f in coefficients),
-
         ufl_assert(isinstance(arguments, Tuple),
             "Expecting Tuple instance with Arguments.")
-        #and all(isinstance(f, Argument) for f in arguments),
-
         ufl_assert(isinstance(coefficient_derivatives, (dict, Data)),
                    "Expecting a dict for coefficient derivatives.")
 
         if isinstance(integrand, Zero):
-            return Zero()
+            return integrand
         return Derivative.__new__(cls)
 
     def __init__(self, integrand, coefficients, arguments, coefficient_derivatives):
@@ -84,16 +75,13 @@ class CoefficientDerivative(Derivative):
         return (self._integrand, self._coefficients, self._arguments, self._coefficient_derivatives)
 
     def shape(self):
-        # Assertion in __new__ guarantees this
-        return ()
+        return self._integrand.shape()
 
     def free_indices(self):
-        # Assertion in __new__ guarantees this
-        return ()
+        return self._integrand.free_indices()
 
     def index_dimensions(self):
-        # Assertion in __new__ guarantees this
-        return {}
+        return self._integrand.index_dimensions()
 
     def __str__(self):
         return "d/dfj { %s }, with fh=%s, dfh/dfj = %s, and coefficient derivatives %s"\
@@ -110,87 +98,11 @@ def split_indices(expression, idx):
     fi = unique_indices(expression.free_indices() + (idx,))
     return fi, idims
 
-class SpatialDerivative(Derivative):
-    "Partial derivative of an expression w.r.t. spatial directions given by indices."
-    __slots__ = ("_expression", "_index", "_shape", "_free_indices", "_index_dimensions", "_repr")
-    def __new__(cls, expression, index):
-        # Return zero if expression is trivially constant
-        if expression.is_cellwise_constant():
-            if isinstance(index, (tuple, MultiIndex)):
-                index, = index
-            fi, idims = split_indices(expression, index)
-            return Zero(expression.shape(), fi, idims)
-        return Derivative.__new__(cls)
-
-    def __init__(self, expression, index):
-        Derivative.__init__(self)
-        self._expression = expression
-
-        # Make a MultiIndex with knowledge of the dimensions
-        sh = (expression.geometric_dimension(),)
-        self._index = as_multi_index(index, sh)
-
-        # Make sure we have a single valid index
-        ufl_assert(len(self._index) == 1, "Expecting a single index.")
-        fi, idims = split_indices(expression, self._index[0])
-
-        # Store what we need
-        self._free_indices = fi
-        self._index_dimensions = idims
-        self._shape = expression.shape()
-
-        self._repr = "SpatialDerivative(%r, %r)" % (self._expression, self._index)
-
-    def operands(self):
-        return (self._expression, self._index)
-
-    def free_indices(self):
-        return self._free_indices
-
-    def index_dimensions(self):
-        return self._index_dimensions
-
-    def shape(self):
-        return self._shape
-
-    def is_cellwise_constant(self):
-        "Return whether this expression is spatially constant over each cell."
-        return self._expression.is_cellwise_constant()
-
-    def evaluate(self, x, mapping, component, index_values, derivatives=()):
-        "Get child from mapping and return the component asked for."
-
-        i = self._index[0]
-
-        if isinstance(i, FixedIndex):
-            i = int(i)
-
-        pushed = False
-        if isinstance(i, Index):
-            i = index_values[i]
-            pushed = True
-            index_values.push(self._index[0], None)
-
-        result = self._expression.evaluate(x, mapping, component, index_values, derivatives=derivatives + (i,))
-
-        if pushed:
-            index_values.pop()
-
-        return result
-
-    def __str__(self):
-        if isinstance(self._expression, Terminal):
-            return "d%s/dx_%s" % (self._expression, self._index)
-        return "d/dx_%s %s" % (self._index, parstr(self._expression, self))
-
-    def __repr__(self):
-        return self._repr
-
 class VariableDerivative(Derivative):
-    __slots__ = ("_f", "_v", "_free_indices", "_index_dimensions", "_shape", "_repr")
+    __slots__ = ("_f", "_v", "_free_indices", "_index_dimensions", "_shape",)
     def __new__(cls, f, v):
-        # Return zero if expression is trivially independent of Coefficient
-        if isinstance(f, Terminal):# and not isinstance(f, Variable):
+        # Return zero if expression is trivially independent of variable
+        if isinstance(f, Terminal):
             free_indices = tuple(set(f.free_indices()) ^ set(v.free_indices()))
             index_dimensions = mergedicts([f.index_dimensions(), v.index_dimensions()])
             index_dimensions = subdict(index_dimensions, free_indices)
@@ -222,7 +134,6 @@ class VariableDerivative(Derivative):
         self._index_dimensions = dict(fid)
         self._index_dimensions.update(vid)
         self._shape = f.shape() + v.shape()
-        self._repr = "VariableDerivative(%r, %r)" % (self._f, self._v)
 
     def operands(self):
         return (self._f, self._v)
@@ -242,7 +153,7 @@ class VariableDerivative(Derivative):
         return "d/d[%s] %s" % (self._v, parstr(self._f, self))
 
     def __repr__(self):
-        return self._repr
+        return "VariableDerivative(%r, %r)" % (self._f, self._v)
 
 #--- Compound differentiation objects ---
 
@@ -253,7 +164,7 @@ class CompoundDerivative(Derivative):
         Derivative.__init__(self)
 
 class Grad(CompoundDerivative):
-    __slots__ = ("_f", "_dim", "_repr")
+    __slots__ = ("_f", "_dim",)
 
     def __new__(cls, f):
         # Return zero if expression is trivially constant
@@ -262,17 +173,12 @@ class Grad(CompoundDerivative):
             free_indices = f.free_indices()
             index_dimensions = subdict(f.index_dimensions(), free_indices)
             return Zero(f.shape() + (dim,), free_indices, index_dimensions)
-        if dim == 1:
-            return f.dx(0)
         return CompoundDerivative.__new__(cls)
 
     def __init__(self, f):
         CompoundDerivative.__init__(self)
         self._f = f
         self._dim = f.geometric_dimension()
-        ufl_assert(not f.free_indices(),\
-            "Taking gradient of an expression with free indices is not supported.")
-        self._repr = "Grad(%r)" % self._f
 
     def reconstruct(self, op):
         "Return a new object of the same type with new operands."
@@ -281,13 +187,21 @@ class Grad(CompoundDerivative):
                        "Operand shape mismatch in Grad reconstruct.")
             ufl_assert(self._f.free_indices() == op.free_indices(),
                        "Free index mismatch in Grad reconstruct.")
-            index_dimensions = {}
             return Zero(self.shape(), self.free_indices(),
                         self.index_dimensions())
         return self.__class__._uflclass(op)
 
+    def evaluate(self, x, mapping, component, index_values, derivatives=()):
+        "Get child from mapping and return the component asked for."
+        r = len(component)
+        component, i = component[:-1], component[-1]
+        derivatives = derivatives + (i,)
+        result = self._f.evaluate(x, mapping, component, index_values,
+                                  derivatives=derivatives)
+        return result
+
     def operands(self):
-        return (self._f, )
+        return (self._f,)
 
     def free_indices(self):
         return self._f.free_indices()
@@ -296,26 +210,22 @@ class Grad(CompoundDerivative):
         return self._f.index_dimensions()
 
     def shape(self):
-        return  self._f.shape() + (self._dim,)
+        return self._f.shape() + (self._dim,)
 
     def __str__(self):
         return "grad(%s)" % self._f
 
     def __repr__(self):
-        return self._repr
+        return "Grad(%r)" % self._f
 
 class Div(CompoundDerivative):
-    __slots__ = ("_f", "_repr")
+    __slots__ = ("_f",)
 
     def __new__(cls, f):
         ufl_assert(not f.free_indices(), \
             "TODO: Taking divergence of an expression with free indices,"\
             "should this be a valid expression? Please provide examples!")
 
-        if f.rank() == 0:
-            return f.dx(0)
-        #ufl_assert(f.rank() >= 1, "Can't take the divergence of a scalar.")
-
         # Return zero if expression is trivially constant
         if f.is_cellwise_constant():
             return Zero(f.shape()[:-1]) # No free indices asserted above
@@ -325,7 +235,6 @@ class Div(CompoundDerivative):
     def __init__(self, f):
         CompoundDerivative.__init__(self)
         self._f = f
-        self._repr = "Div(%r)" % self._f
 
     def operands(self):
         return (self._f, )
@@ -343,10 +252,10 @@ class Div(CompoundDerivative):
         return "div(%s)" % self._f
 
     def __repr__(self):
-        return self._repr
+        return "Div(%r)" % self._f
 
 class NablaGrad(CompoundDerivative):
-    __slots__ = ("_f", "_dim", "_repr")
+    __slots__ = ("_f", "_dim",)
 
     def __new__(cls, f):
         # Return zero if expression is trivially constant
@@ -355,17 +264,12 @@ class NablaGrad(CompoundDerivative):
             free_indices = f.free_indices()
             index_dimensions = subdict(f.index_dimensions(), free_indices)
             return Zero((dim,) + f.shape(), free_indices, index_dimensions)
-        if dim == 1:
-            return f.dx(0)
         return CompoundDerivative.__new__(cls)
 
     def __init__(self, f):
         CompoundDerivative.__init__(self)
         self._f = f
         self._dim = f.geometric_dimension()
-        ufl_assert(not f.free_indices(),\
-            "Taking gradient of an expression with free indices is not supported.")
-        self._repr = "NablaGrad(%r)" % self._f
 
     def reconstruct(self, op):
         "Return a new object of the same type with new operands."
@@ -374,7 +278,6 @@ class NablaGrad(CompoundDerivative):
                        "Operand shape mismatch in NablaGrad reconstruct.")
             ufl_assert(self._f.free_indices() == op.free_indices(),
                        "Free index mismatch in NablaGrad reconstruct.")
-            index_dimensions = {}
             return Zero(self.shape(), self.free_indices(),
                         self.index_dimensions())
         return self.__class__._uflclass(op)
@@ -395,20 +298,16 @@ class NablaGrad(CompoundDerivative):
         return "nabla_grad(%s)" % self._f
 
     def __repr__(self):
-        return self._repr
+        return "NablaGrad(%r)" % self._f
 
 class NablaDiv(CompoundDerivative):
-    __slots__ = ("_f", "_repr")
+    __slots__ = ("_f",)
 
     def __new__(cls, f):
         ufl_assert(not f.free_indices(), \
             "TODO: Taking divergence of an expression with free indices,"\
             "should this be a valid expression? Please provide examples!")
 
-        if f.rank() == 0:
-            return f.dx(0)
-        #ufl_assert(f.rank() >= 1, "Can't take the divergence of a scalar.")
-
         # Return zero if expression is trivially constant
         if f.is_cellwise_constant():
             return Zero(f.shape()[1:]) # No free indices asserted above
@@ -418,7 +317,6 @@ class NablaDiv(CompoundDerivative):
     def __init__(self, f):
         CompoundDerivative.__init__(self)
         self._f = f
-        self._repr = "NablaDiv(%r)" % self._f
 
     def operands(self):
         return (self._f, )
@@ -436,10 +334,10 @@ class NablaDiv(CompoundDerivative):
         return "nabla_div(%s)" % self._f
 
     def __repr__(self):
-        return self._repr
+        return "NablaDiv(%r)" % self._f
 
 class Curl(CompoundDerivative):
-    __slots__ = ("_f", "_dim", "_repr")
+    __slots__ = ("_f", "_shape",)
 
     def __new__(cls, f):
         # Validate input
@@ -462,7 +360,6 @@ class Curl(CompoundDerivative):
         sh = { (): (2,), (2,): (), (3,): (3,) }[f.shape()]
         self._f = f
         self._shape = sh
-        self._repr = "Curl(%r)" % self._f
 
     def operands(self):
         return (self._f, )
@@ -480,5 +377,4 @@ class Curl(CompoundDerivative):
         return "curl(%s)" % self._f
 
     def __repr__(self):
-        return self._repr
-
+        return "Curl(%r)" % self._f
