@@ -1,6 +1,6 @@
 """This module defines the Indexed class."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -20,18 +20,20 @@
 # First added:  2009-01-28
 # Last changed: 2011-06-17
 
-from collections import defaultdict
+from itertools import izip
 from ufl.log import error
-from ufl.expr import Expr, WrapperType
-from ufl.indexing import IndexBase, Index, FixedIndex, as_multi_index
+from ufl.expr import Expr
+from ufl.operatorbase import WrapperType
+from ufl.indexing import Index, FixedIndex, as_multi_index
 from ufl.indexutils import unique_indices
 from ufl.precedence import parstr
+from ufl.common import EmptyDict
 
 #--- Indexed expression ---
 
 class Indexed(WrapperType):
     __slots__ = ("_expression", "_indices",
-                 "_free_indices", "_index_dimensions", "_repr")
+                 "_free_indices", "_index_dimensions",)
     def __init__(self, expression, indices):
         WrapperType.__init__(self)
         if not isinstance(expression, Expr):
@@ -46,31 +48,30 @@ class Indexed(WrapperType):
 
         self._indices = as_multi_index(indices, shape)
 
-        for si, di in zip(shape, self._indices):
+        for si, di in izip(shape, self._indices):
             if isinstance(di, FixedIndex) and int(di) >= int(si):
                 error("Fixed index out of range!")
 
-        idims = dict((i, s) for (i, s) in zip(self._indices._indices, shape) if isinstance(i, Index))
+        idims = dict((i, s) for (i, s) in izip(self._indices._indices, shape)
+                     if isinstance(i, Index))
         idims.update(expression.index_dimensions())
         fi = unique_indices(expression.free_indices() + self._indices._indices)
-        
-        self._free_indices = fi
-        self._index_dimensions = idims
 
-        self._repr = "Indexed(%r, %r)" % (self._expression, self._indices)
-    
+        self._free_indices = fi
+        self._index_dimensions = idims or EmptyDict
+
     def operands(self):
         return (self._expression, self._indices)
-    
+
     def free_indices(self):
         return self._free_indices
-    
+
     def index_dimensions(self):
         return self._index_dimensions
-    
+
     def shape(self):
         return ()
-    
+
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
         return self._expression.is_cellwise_constant()
@@ -80,14 +81,17 @@ class Indexed(WrapperType):
         component = ii.evaluate(x, mapping, None, index_values)
         if derivatives:
             return A.evaluate(x, mapping, component, index_values, derivatives)
-        return A.evaluate(x, mapping, component, index_values)
+        else:
+            return A.evaluate(x, mapping, component, index_values)
 
     def __str__(self):
         return "%s[%s]" % (parstr(self._expression, self), self._indices)
-    
+
     def __repr__(self):
-        return self._repr
-    
+        return "Indexed(%r, %r)" % (self._expression, self._indices)
+
     def __getitem__(self, key):
         error("Attempting to index with %r, but object is already indexed: %r" % (key, self))
 
+    def __getnewargs__(self):
+        return ()
