@@ -1,6 +1,6 @@
 """This module provides basic mathematical functions."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,25 +21,18 @@
 # Modified by Kristian B. Oelgaard, 2011
 #
 # First added:  2008-03-14
-# Last changed: 2011-10-25
+# Last changed: 2013-03-15
 
 import math
 from ufl.log import warning, error
 from ufl.assertions import ufl_assert
-from ufl.expr import Operator
-from ufl.constantvalue import is_true_ufl_scalar, ScalarValue, Zero, FloatValue, IntValue
+from ufl.operatorbase import Operator
+from ufl.constantvalue import is_true_ufl_scalar, ScalarValue, Zero, FloatValue, IntValue, as_ufl
+from ufl.common import EmptyDict
 
 """
 TODO: Include additional functions available in <cmath> (need derivatives as well):
 
-Trigonometric functions:
-atan2    Compute arc tangent with two parameters (function)
-
-Hyperbolic functions:
-cosh     Compute hyperbolic cosine (function)
-sinh     Compute hyperbolic sine (function)
-tanh     Compute hyperbolic tangent (function)
-
 Exponential and logarithmic functions:
 log10    Compute common logarithm (function)
 
@@ -63,27 +56,26 @@ Implementation in C++ std::tr1:: or boost::math::tr1::
 class MathFunction(Operator):
     "Base class for all math functions"
     # Freeze member variables for objects in this class
-    __slots__ = ("_name", "_argument", "_repr")
+    __slots__ = ("_name", "_argument",)
     def __init__(self, name, argument):
         Operator.__init__(self)
         ufl_assert(is_true_ufl_scalar(argument), "Expecting scalar argument.")
         self._name     = name
         self._argument = argument
-        self._repr = "%s(%r)" % (name, argument)
-    
+
     def operands(self):
         return (self._argument,)
-    
+
     def free_indices(self):
         return ()
-    
+
     def index_dimensions(self):
-        return {}
-    
+        return EmptyDict
+
     def shape(self):
         return ()
-    
-    def evaluate(self, x, mapping, component, index_values):    
+
+    def evaluate(self, x, mapping, component, index_values):
         a = self._argument.evaluate(x, mapping, component, index_values)
         try:
             res = getattr(math, self._name)(a)
@@ -91,98 +83,181 @@ class MathFunction(Operator):
             warning('Value error in evaluation of function %s with argument %s.' % (self._name, a))
             raise
         return res
-    
+
     def __str__(self):
         return "%s(%s)" % (self._name, self._argument)
-    
+
     def __repr__(self):
-        return self._repr
+        return "%s(%r)" % (self._name, self._argument)
 
 class Sqrt(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.sqrt(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "sqrt", argument)
 
 class Exp(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.exp(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "exp", argument)
 
 class Ln(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.log(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "ln", argument)
-    
-    def evaluate(self, x, mapping, component, index_values):    
+
+    def evaluate(self, x, mapping, component, index_values):
         a = self._argument.evaluate(x, mapping, component, index_values)
         return math.log(a)
 
 class Cos(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.cos(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "cos", argument)
 
 class Sin(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.sin(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "sin", argument)
 
 class Tan(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.tan(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "tan", argument)
 
+class Cosh(MathFunction):
+    __slots__ = ()
+    def __new__(cls, argument):
+        if isinstance(argument, (ScalarValue, Zero)):
+            return FloatValue(math.cosh(float(argument)))
+        return MathFunction.__new__(cls)
+
+    def __init__(self, argument):
+        MathFunction.__init__(self, "cosh", argument)
+
+class Sinh(MathFunction):
+    __slots__ = ()
+    def __new__(cls, argument):
+        if isinstance(argument, (ScalarValue, Zero)):
+            return FloatValue(math.sinh(float(argument)))
+        return MathFunction.__new__(cls)
+
+    def __init__(self, argument):
+        MathFunction.__init__(self, "sinh", argument)
+
+class Tanh(MathFunction):
+    __slots__ = ()
+    def __new__(cls, argument):
+        if isinstance(argument, (ScalarValue, Zero)):
+            return FloatValue(math.tanh(float(argument)))
+        return MathFunction.__new__(cls)
+
+    def __init__(self, argument):
+        MathFunction.__init__(self, "tanh", argument)
+
 class Acos(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.acos(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "acos", argument)
 
 class Asin(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.asin(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "asin", argument)
 
 class Atan(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.atan(float(argument)))
         return MathFunction.__new__(cls)
-    
+
     def __init__(self, argument):
         MathFunction.__init__(self, "atan", argument)
 
+class Atan2(Operator):
+    __slots__ = ("_name", "_arg1", "_arg2", "_classname")
+    def __new__(cls, arg1, arg2):
+        if isinstance(arg1, (ScalarValue, Zero)) and isinstance(arg2, (ScalarValue, Zero)):
+            return FloatValue(math.atan2(float(arg1), float(arg2)))
+        return Operator.__new__(cls)
+
+    def __init__(self, arg1, arg2):
+        Operator.__init__(self)
+        ufl_assert(is_true_ufl_scalar(arg1), "Expecting scalar argument 1.")
+        ufl_assert(is_true_ufl_scalar(arg2), "Expecting scalar argument 2.")
+        self._name     = "atan_2"
+        self._arg1 = arg1
+        self._arg2 = arg2
+
+    def operands(self):
+        return (self._arg1, self._arg2)
+
+    def free_indices(self):
+        return ()
+
+    def index_dimensions(self):
+        return EmptyDict
+
+    def shape(self):
+        return ()
+
+    def evaluate(self, x, mapping, component, index_values):
+        a = self._arg1.evaluate(x, mapping, component, index_values)
+        b = self._arg2.evaluate(x, mapping, component, index_values)
+        try:
+            res = math.atan2(a,b)
+        except ValueError:
+            warning('Value error in evaluation of function %s with arguments %s, %s.' % (self._name, a,b))
+            raise
+        return res
+
+    def __str__(self):
+        return "%s(%s,%s)" % (self._name, self._arg1, self._arg2)
+
+    def __repr__(self):
+        return "%s(%s,%s)" % (self._name, self._arg1, self._arg2)
+
+
 def _find_erf():
     import math
     if hasattr(math, 'erf'):
@@ -193,6 +268,7 @@ def _find_erf():
     return None
 
 class Erf(MathFunction):
+    __slots__ = ()
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             erf = _find_erf()
@@ -203,7 +279,7 @@ class Erf(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "erf", argument)
 
-    def evaluate(self, x, mapping, component, index_values):    
+    def evaluate(self, x, mapping, component, index_values):
         a = self._argument.evaluate(x, mapping, component, index_values)
         erf = _find_erf()
         if erf is None:
@@ -213,15 +289,21 @@ class Erf(MathFunction):
 class BesselFunction(Operator):
     "Base class for all bessel functions"
     # Freeze member variables for objects in this class
-    __slots__ = ("_name", "_nu", "_argument", "_repr")
+    __slots__ = ("_name", "_nu", "_argument", "_classname")
     def __init__(self, name, classname, nu, argument):
         Operator.__init__(self)
-        ufl_assert(is_true_ufl_scalar(nu), "Expecting scalar argument.")
+        ufl_assert(is_true_ufl_scalar(nu), "Expecting scalar nu.")
         ufl_assert(is_true_ufl_scalar(argument), "Expecting scalar argument.")
+        fnu = float(nu)
+        inu = int(nu)
+        if fnu == inu:
+            nu = as_ufl(inu)
+        else:
+            nu = as_ufl(fnu)
+        self._classname = classname
         self._name     = name
         self._nu       = nu
         self._argument = argument
-        self._repr = "%s(%r, %r)" % (classname, nu, argument)
 
     def operands(self):
         return (self._nu, self._argument)
@@ -230,12 +312,12 @@ class BesselFunction(Operator):
         return ()
 
     def index_dimensions(self):
-        return {}
+        return EmptyDict
 
     def shape(self):
         return ()
 
-    def evaluate(self, x, mapping, component, index_values):    
+    def evaluate(self, x, mapping, component, index_values):
         a = self._argument.evaluate(x, mapping, component, index_values)
         try:
             import scipy.special
@@ -255,20 +337,24 @@ class BesselFunction(Operator):
         return "%s(%s, %s)" % (self._name, self._nu, self._argument)
 
     def __repr__(self):
-        return self._repr
+        return "%s(%r, %r)" % (self._classname, self._nu, self._argument)
 
 class BesselJ(BesselFunction):
+    __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_j", "BesselJ", nu, argument)
 
 class BesselY(BesselFunction):
+    __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_y", "BesselY", nu, argument)
 
 class BesselI(BesselFunction):
+    __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_i", "BesselI", nu, argument)
 
 class BesselK(BesselFunction):
+    __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_k", "BesselK", nu, argument)
