@@ -1,6 +1,6 @@
 "Utility objects for pretty syntax in user code."
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,39 +21,34 @@
 # Modified by Kristian Oelgaard, 2009
 #
 # First added:  2008-03-14
-# Last changed: 2011-06-02
+# Last changed: 2013-01-11
 
 from ufl.indexing import indices
 from ufl.integral import Measure
-from ufl.geometry import Space, Cell
+from ufl.geometry import Cell
 
 # Default indices
 i, j, k, l = indices(4)
 p, q, r, s = indices(4)
 
 # Default measures for integration
-dx = Measure(Measure.CELL, 0)
-ds = Measure(Measure.EXTERIOR_FACET, 0)
-dS = Measure(Measure.INTERIOR_FACET, 0)
-dE = Measure(Measure.MACRO_CELL, 0)
-dc = Measure(Measure.SURFACE, 0)
-
-# Euclidean spaces
-R0 = Space(0)
-R1 = Space(1)
-R2 = Space(2)
-R3 = Space(3)
+dx = Measure(Measure.CELL,           Measure.DOMAIN_ID_DEFAULT)
+ds = Measure(Measure.EXTERIOR_FACET, Measure.DOMAIN_ID_DEFAULT)
+dS = Measure(Measure.INTERIOR_FACET, Measure.DOMAIN_ID_DEFAULT)
+dP = Measure(Measure.POINT,          Measure.DOMAIN_ID_DEFAULT)
+dE = Measure(Measure.MACRO_CELL,     Measure.DOMAIN_ID_DEFAULT)
+dc = Measure(Measure.SURFACE,        Measure.DOMAIN_ID_DEFAULT)
 
 # Cell types
-cell1D        = Cell("cell1D", R1)
-cell2D        = Cell("cell2D", R2)
-cell3D        = Cell("cell3D", R3)
-vertex        = Cell("vertex", R0)
-interval      = Cell("interval", R1)
-triangle      = Cell("triangle", R2)
-tetrahedron   = Cell("tetrahedron", R3)
-quadrilateral = Cell("quadrilateral", R2)
-hexahedron    = Cell("hexahedron", R3)
+cell1D        = Cell("cell1D", 1)
+cell2D        = Cell("cell2D", 2)
+cell3D        = Cell("cell3D", 3)
+vertex        = Cell("vertex", 0)
+interval      = Cell("interval", 1)
+triangle      = Cell("triangle", 2)
+tetrahedron   = Cell("tetrahedron", 3)
+quadrilateral = Cell("quadrilateral", 2)
+hexahedron    = Cell("hexahedron", 3)
 
 # Facet is just a dummy declaration for RestrictedElement
 facet = "facet"
