@@ -3,7 +3,7 @@ which are either already available as member functions on UFL objects
 or defined as compound operators involving basic operations on the UFL
 objects."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2013 Martin Sandve Alnes and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -23,9 +23,8 @@ objects."""
 # Modified by Kristian B. Oelgaard, 2011
 #
 # First added:  2008-04-09
-# Last changed: 2011-10-24
+# Last changed: 2013-03-15
 
-import math
 import operator
 from ufl.log import error, warning
 from ufl.assertions import ufl_assert
@@ -39,9 +38,11 @@ from ufl.tensors import as_tensor, as_matrix, as_vector, ListTensor
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, \
     AndCondition, OrCondition, NotCondition, Conditional
 from ufl.mathfunctions import Sqrt, Exp, Ln, Erf,\
-    Cos, Sin, Tan, Acos, Asin, Atan,\
+    Cos, Sin, Tan, Cosh, Sinh, Tanh, Acos, Asin, Atan, Atan2,\
     BesselJ, BesselY, BesselI, BesselK
-from ufl.indexing import indices, Indexed
+from ufl.restriction import CellAvg, FacetAvg
+from ufl.indexing import indices
+from ufl.indexed import Indexed
 from ufl.geometry import SpatialCoordinate
 
 #--- Basic operators ---
@@ -102,8 +103,16 @@ def transpose(A):
         return A
     return Transposed(A)
 
-def outer(a, b):
-    "UFL operator: Take the outer product of a and b."
+def outer(*operands):
+    "UFL operator: Take the outer product of two or more operands."
+    n = len(operands)
+    if n == 1:
+        return operands[0]
+    elif n == 2:
+        a, b = operands
+    else:
+        a = outer(*operands[:-1])
+        b = operands[-1]
     a = as_ufl(a)
     b = as_ufl(b)
     if a.shape() == () and b.shape() == ():
@@ -156,6 +165,12 @@ def contraction(a, a_axes, b, b_axes):
     ii = tuple(i for i in (aii + bii) if not i in cii)
     return as_tensor(s, ii)
 
+def perp(v):
+    "UFL operator: Take the perp of v, i.e. (-v1, +v0)."
+    v = as_ufl(v)
+    ufl_assert(v.shape() == (2,), "Expecting a 2D vector expression.")
+    return as_vector((-v[1],v[0]))
+
 def cross(a, b):
     "UFL operator: Take the cross product of a and b."
     a = as_ufl(a)
@@ -191,7 +206,7 @@ def tr(A):
 def diag(A):
     """UFL operator: Take the diagonal part of rank 2 tensor A _or_
     make a diagonal rank 2 tensor from a rank 1 tensor.
-    
+
     Always returns a rank 2 tensor. See also diag_vector."""
 
     # TODO: Make a compound type or two for this operator
@@ -215,8 +230,8 @@ def diag(A):
     return as_matrix(rows)
 
 def diag_vector(A):
-    """UFL operator: Take the diagonal part of rank 2 tensor A and return as a vector."
-    
+    """UFL operator: Take the diagonal part of rank 2 tensor A and return as a vector.
+
     See also diag."""
 
     # TODO: Make a compound type for this operator
@@ -247,7 +262,8 @@ def sym(A):
 #--- Differential operators
 
 def Dx(f, *i):
-    "UFL operator: Take the partial derivative of f with respect to spatial variable number i. Equivalent to f.dx(\*i)."
+    """UFL operator: Take the partial derivative of f with respect
+    to spatial variable number i. Equivalent to f.dx(\*i)."""
     f = as_ufl(f)
     return f.dx(*i)
 
@@ -257,10 +273,11 @@ def Dt(f):
     raise NotImplementedError
 
 def Dn(f):
-    "UFL operator: Take the directional derivative of f in the facet normal direction, Dn(f) := dot(grad(f), n)."
+    """UFL operator: Take the directional derivative of f in the
+    facet normal direction, Dn(f) := dot(grad(f), n)."""
     f = as_ufl(f)
     cell = f.cell()
-    if cell is None:
+    if cell is None: # FIXME: Rather if f.is_cellwise_constant()?
         return Zero(f.shape(), f.free_indices(), f.index_dimensions())
     return dot(grad(f), cell.n)
 
@@ -270,25 +287,21 @@ def diff(f, v):
     If f is a form, diff is applied to each integrand.
     """
     if isinstance(f, Form):
-        from ufl.algorithms.transformations import transform_integrands
-        def _diff(e):
-            return diff(e, v)
-        return transform_integrands(f, _diff)
+        from ufl.algorithms.transformer import transform_integrands
+        return transform_integrands(f, lambda e: diff(e, v))
     else:
         f = as_ufl(f)
 
     if isinstance(v, SpatialCoordinate):
-        if v.shape() == ():
-            return f.dx(0)
-        else:
-            r = f.rank()
-            ii = indices(r + 1)
-            if r:
-                f = f[ii[:-1]]
-            df = f.dx(ii[-1])
-            return as_tensor(df, ii)
-
-    return VariableDerivative(f, v)
+        return grad(f)
+    # TODO: Allow this? Must be tested well!
+    #elif (isinstance(v, Indexed)
+    #      and isinstance(v.operands()[0], SpatialCoordinate)):
+    #    return grad(f)[...,v.operands()[1]]
+    elif isinstance(v, Variable):
+        return VariableDerivative(f, v)
+    else:
+        error("Expecting a Variable or SpatialCoordinate in diff.")
 
 def grad(f):
     """UFL operator: Take the gradient of f.
@@ -379,18 +392,26 @@ def jump(v, n=None):
         r = v.rank()
         if r == 0:
             return v('+')*n('+') + v('-')*n('-')
-        elif r == 1:
+        else:
             return dot(v('+'), n('+')) + dot(v('-'), n('-'))
-        elif r == 2:
-            return dot(v('+'), n('+')) + dot(v('-'), n('-'))
-
-    error("jump(v, n) is only defined for scalar, vector, or matrix-valued expressions (not rank %d expressions)." % r)
 
 def avg(v):
     "UFL operator: Take the average of v across a facet."
     v = as_ufl(v)
     return 0.5*(v('+') + v('-'))
 
+def cell_avg(f):
+    "UFL operator: Take the average of v over a cell."
+    #ufl_assert((isinstance(f, Restricted) and isinstance(f.operands()[0], FormArgument)) or
+    #    isinstance(f, FormArgument), "Can only take the cell average of a (optionally restricted) Coefficient or Argument.")
+    return CellAvg(f)
+
+def facet_avg(f):
+    "UFL operator: Take the average of v over a facet."
+    #ufl_assert((isinstance(f, Restricted) and isinstance(f.operands()[0], FormArgument)) or
+    #    isinstance(f, FormArgument), "Can only take the cell average of a (optionally restricted) Coefficient or Argument.")
+    return FacetAvg(f)
+
 #--- Other operators ---
 
 def variable(e):
@@ -415,19 +436,19 @@ def ne(left, right):
 
 def le(left, right):
     "UFL operator: A boolean expresion (left <= right) for use with conditional."
-    return LE(left, right)
+    return as_ufl(left) <= as_ufl(right)
 
 def ge(left, right):
     "UFL operator: A boolean expresion (left >= right) for use with conditional."
-    return GE(left, right)
+    return as_ufl(left) >= as_ufl(right)
 
 def lt(left, right):
     "UFL operator: A boolean expresion (left < right) for use with conditional."
-    return LT(left, right)
+    return as_ufl(left) < as_ufl(right)
 
 def gt(left, right):
     "UFL operator: A boolean expresion (left > right) for use with conditional."
-    return GT(left, right)
+    return as_ufl(left) > as_ufl(right)
 
 def And(left, right):
     "UFL operator: A boolean expresion (left and right) for use with conditional."
@@ -446,6 +467,16 @@ def sign(x):
     # TODO: Add a Sign type for this?
     return conditional(eq(x, 0), 0, conditional(lt(x, 0), -1, +1))
 
+def Max(x, y):
+    "UFL operator: Take the maximum of x and y."
+    # TODO: Add a Maximum type for this?
+    return conditional(gt(x, y), x, y)
+
+def Min(x, y):
+    "UFL operator: Take the minimum of x and y."
+    # TODO: Add a Minimum type for this?
+    return conditional(lt(x, y), x, y)
+
 #--- Math functions ---
 
 def _mathfunction(f, cls):
@@ -479,6 +510,18 @@ def tan(f):
     "UFL operator: Take the tangent of f."
     return _mathfunction(f, Tan)
 
+def cosh(f):
+    "UFL operator: Take the cosinus hyperbolicus of f."
+    return _mathfunction(f, Cosh)
+
+def sinh(f):
+    "UFL operator: Take the sinus hyperbolicus of f."
+    return _mathfunction(f, Sinh)
+
+def tanh(f):
+    "UFL operator: Take the tangent hyperbolicus of f."
+    return _mathfunction(f, Tanh)
+
 def acos(f):
     "UFL operator: Take the inverse cosinus of f."
     return _mathfunction(f, Acos)
@@ -491,6 +534,15 @@ def atan(f):
     "UFL operator: Take the inverse tangent of f."
     return _mathfunction(f, Atan)
 
+def atan_2(f1,f2):
+    "UFL operator: Take the inverse tangent of f."
+    f1 = as_ufl(f1)
+    f2 = as_ufl(f2)
+    r = Atan2(f1, f2)
+    if isinstance(r, (ScalarValue, Zero, int, float)):
+        return float(r)
+    return r
+
 def erf(f):
     "UFL operator: Take the error function of f."
     return _mathfunction(f, Erf)
@@ -532,25 +584,28 @@ def exterior_derivative(f):
     as opposed to 'nabla_grad' and 'nabla_div'.
     """
 
+    # Extract the element from the input f
     if isinstance(f, Indexed):
         if len(f._indices) > 1:
             raise NotImplementedError
         index = int(f._indices[0])
         element = f._expression.element()
-        subelement = element.extract_component(index)[1]
-        family = subelement.family()
+        element = element.extract_component(index)[1]
     elif isinstance(f, ListTensor):
         if len(f._expressions[0]._indices) > 1:
             raise NotImplementedError
         index = int(f._expressions[0]._indices[0])
         element = f._expressions[0]._expression.element()
-        subelement = element.extract_component(index)[1]
-        family = subelement.family()
+        element = element.extract_component(index)[1]
     else:
         try:
-            family = f.element().family()
+            element = f.element()
         except:
-            error("Unable to determine element family from %s" % f)
+            error("Unable to determine element from %s" % f)
+
+    # Extract the family and the
+    family = element.family()
+    gdim = element.cell().geometric_dimension()
 
     # L^2 elements:
     if "Disc" in family:
@@ -558,6 +613,8 @@ def exterior_derivative(f):
 
     # H^1 elements:
     if "Lagrange" in family:
+        if gdim == 1:
+            return grad(f)[0]  # Special-case 1D vectors as scalars
         return grad(f)
 
     # H(curl) elements:
@@ -569,4 +626,3 @@ def exterior_derivative(f):
         return div(f)
 
     error("Unable to determine exterior_derivative. Family is '%s'" % family)
-
