@@ -99,7 +99,7 @@ def build_precedence_mapping(precedence_list):
 def assign_precedences(precedence_list):
     "Given a precedence list, assign ints to class._precedence."
     pm, missing = build_precedence_mapping(precedence_list)
-    for c, p in pm.iteritems():
+    for c, p in sorted(pm.iteritems(), key=lambda x: x[0].__name__):
         c._precedence = p
     if missing:
         msg = "Missing precedence levels for classes:\n" +\
