@@ -1,6 +1,6 @@
 "Algorithm for splitting a Coefficient or Argument into subfunctions."
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -11,27 +11,27 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008
 #
 # First added:  2008-03-14
-# Last changed: 2009-03-13
+# Last changed: 2011-06-22
 
 from ufl.log import error
 from ufl.assertions import ufl_assert
-from ufl.common import product
+from ufl.common import product, EmptyDict
 from ufl.finiteelement import MixedElement, TensorElement
 from ufl.tensors import as_vector, as_matrix, as_tensor
 
 
 def split(v):
-    "Split Coefficient or Argument into its subfunctions if any."
-
+    """UFL operator: If v is a Coefficient or Argument in a mixed space, returns
+    a tuple with the function components corresponding to the subelements."""
     # Special case: simple element, just return function in a tuple
     element = v.element()
     if not isinstance(element, MixedElement):
@@ -70,16 +70,15 @@ def split(v):
 
         elif rank == 2:
             # This subelement is a tensor, possibly with symmetries, slightly more complicated...
-            
+
             # Size of this subvalue
             sub_size = product(shape)
-            
+
             # If this subelement is a symmetric element, subtract symmetric components
             s = None
             if isinstance(e, TensorElement):
                 s = e.symmetry()
-            if s is None:
-                s = {}
+            s = s or EmptyDict
             # If we do this, we must fix the size computation in MixedElement.__init__ as well
             #actual_value_size -= len(s)
             #sub_size -= len(s)
@@ -101,13 +100,13 @@ def split(v):
                         #print "k, offset, i, j, shape, component", k, offset, i, j, shape, component
                     elif v.rank() == 2:
                         # Mapping into a concatenated tensor (is this a figment of my imagination?)
-                        error("Not implemented. Please send a minimal test that fails here to ufl-dev@fenics.org.")
+                        error("Not implemented.")
                         row_offset, col_offset = 0, 0 # TODO
                         k = (row_offset + i, col_offset + j)
                         component = v[k]
                     row.append(component)
                 components.append(row)
-            
+
             # Make a matrix of the components
             subv = as_matrix(components)
             offset += sub_size
@@ -116,12 +115,11 @@ def split(v):
             # TODO: Handle rank > 2? Or is there such a thing?
             error("Don't know how to split functions with sub functions of rank %d (yet)." % rank)
             #for indices in compute_indices(shape):
-            #    #k = offset + sum(i*s for (i,s) in zip(indices, shape[1:] + (1,)))
+            #    #k = offset + sum(i*s for (i,s) in izip(indices, shape[1:] + (1,)))
             #    vs.append(v[indices])
 
         sub_functions.append(subv)
 
     ufl_assert(actual_value_size == offset, "Logic breach in function splitting.")
-    
-    return tuple(sub_functions)
 
+    return tuple(sub_functions)
