@@ -1,6 +1,6 @@
 """Compound tensor algebra operations."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,15 +18,15 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2008-03-14
-# Last changed: 2011-06-02
+# Last changed: 2013-01-02
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
-from ufl.expr import Expr
-from ufl.constantvalue import ConstantValue, Zero
-from ufl.indexing import Index, indices
+from ufl.constantvalue import Zero
 from ufl.algebra import AlgebraOperator
 from ufl.precedence import parstr
+from ufl.sorting import sorted_expr
+from ufl.common import EmptyDict
 
 def merge_indices(a, b):
     ai = a.free_indices()
@@ -35,13 +35,13 @@ def merge_indices(a, b):
     ri = set(ai) & set(bi)
     ufl_assert(not ri, "Not expecting repeated indices.")
     free_indices = ai+bi
-    
+
     aid = a.index_dimensions()
     bid = b.index_dimensions()
     index_dimensions = dict(aid)
     index_dimensions.update(bid)
-    
-    return free_indices, index_dimensions 
+
+    return free_indices, index_dimensions
 
 ### Algebraic operations on tensors:
 # FloatValues:
@@ -71,12 +71,13 @@ def merge_indices(a, b):
 # --- Classes representing compound tensor algebra operations ---
 
 class CompoundTensorOperator(AlgebraOperator):
+    __slots__ = ()
     def __init__(self):
         AlgebraOperator.__init__(self)
 
 # TODO: Use this and make Sum handle scalars only?
 #       This would simplify some algorithms. The only
-#       problem is we can't use + in many algorithms because 
+#       problem is we can't use + in many algorithms because
 #       this type should be expanded by expand_compounds.
 #class TensorSum(CompoundTensorOperator):
 #    "Sum of nonscalar expressions."
@@ -84,7 +85,7 @@ class CompoundTensorOperator(AlgebraOperator):
 
 # TODO: Use this similarly to TensorSum?
 #       This would simplify some algorithms. The only
-#       problem is we can't use / in many algorithms because 
+#       problem is we can't use / in many algorithms because
 #       this type should be expanded by expand_compounds.
 #class TensorDivision(CompoundTensorOperator):
 #    "Division of nonscalar expression with a scalar expression."
@@ -92,7 +93,7 @@ class CompoundTensorOperator(AlgebraOperator):
 
 # TODO: Use this similarly to TensorSum?
 #       This would simplify some algorithms. The only
-#       problem is we can't use * in many algorithms because 
+#       problem is we can't use * in many algorithms because
 #       this type should be expanded by expand_compounds.
 #class MatrixProduct(CompoundTensorOperator):
 #    "Product of a matrix with a matrix or vector."
@@ -100,7 +101,7 @@ class CompoundTensorOperator(AlgebraOperator):
 
 # TODO: Use this similarly to TensorSum?
 #       This would simplify some algorithms. The only
-#       problem is we can't use abs in many algorithms because 
+#       problem is we can't use abs in many algorithms because
 #       this type should be expanded by expand_compounds.
 #class TensorAbs(CompoundTensorOperator):
 #    "Absolute value of nonscalar expression."
@@ -114,28 +115,28 @@ class Transposed(CompoundTensorOperator):
             a, b = A.shape()
             return Zero((b, a), A.free_indices(), A.index_dimensions())
         return CompoundTensorOperator.__new__(cls)
-    
+
     def __init__(self, A):
         CompoundTensorOperator.__init__(self)
         ufl_assert(A.rank() == 2, "Transposed is only defined for rank 2 tensors.")
         self._A = A
-    
+
     def operands(self):
         return (self._A,)
-    
+
     def free_indices(self):
         return self._A.free_indices()
-    
+
     def index_dimensions(self):
         return self._A.index_dimensions()
-    
+
     def shape(self):
         s = self._A.shape()
         return (s[1], s[0])
-    
+
     def __str__(self):
         return "%s^T" % parstr(self._A, self)
-    
+
     def __repr__(self):
         return "Transposed(%r)" % self._A
 
@@ -156,22 +157,22 @@ class Outer(CompoundTensorOperator):
         self._a = a
         self._b = b
         self._free_indices, self._index_dimensions = merge_indices(a, b)
-    
+
     def operands(self):
         return (self._a, self._b)
-    
+
     def free_indices(self):
         return self._free_indices
-    
+
     def index_dimensions(self):
         return self._index_dimensions
-    
+
     def shape(self):
         return self._a.shape() + self._b.shape()
-    
+
     def __str__(self):
         return "%s (X) %s" % (parstr(self._a, self), parstr(self._b, self))
-    
+
     def __repr__(self):
         return "Outer(%r, %r)" % (self._a, self._b)
 
@@ -190,27 +191,34 @@ class Inner(CompoundTensorOperator):
 
     def __init__(self, a, b):
         CompoundTensorOperator.__init__(self)
-        # sort operands by their repr TODO: This may be slow, can we do better? Needs to be completely independent of the outside world.
-        a, b = sorted((a,b), key = lambda x: repr(x))
+
+        # sort operands for unique representation,
+        # must be independent of various counts etc.
+        # as explained in cmp_expr
+        a, b = sorted_expr((a,b))
+
+        # old version, slow and unsafe:
+        #a, b = sorted((a,b), key = lambda x: repr(x))
+
         self._a = a
         self._b = b
         self._free_indices, self._index_dimensions = merge_indices(a, b)
-    
+
     def operands(self):
         return (self._a, self._b)
-    
+
     def free_indices(self):
         return self._free_indices
-    
+
     def index_dimensions(self):
         return self._index_dimensions
-    
+
     def shape(self):
         return ()
-    
+
     def __str__(self):
         return "%s : %s" % (parstr(self._a, self), parstr(self._b, self))
-    
+
     def __repr__(self):
         return "Inner(%r, %r)" % (self._a, self._b)
 
@@ -218,16 +226,20 @@ class Dot(CompoundTensorOperator):
     __slots__ = ("_a", "_b", "_free_indices", "_index_dimensions")
 
     def __new__(cls, a, b):
-        ar, br = a.rank(), b.rank()
-        ufl_assert((ar >= 1 and br >= 1) or (ar == 0 and br == 0),
+        ash = a.shape()
+        bsh = b.shape()
+        ar, br = len(ash), len(bsh)
+        scalar = (ar == 0 and br == 0)
+        ufl_assert((ar >= 1 and br >= 1) or scalar,
             "Dot product requires non-scalar arguments, "\
             "got arguments with ranks %d and %d." % \
             (ar, br))
+        ufl_assert(scalar or ash[-1] == bsh[0], "Dimension mismatch in dot product.")
         if isinstance(a, Zero) or isinstance(b, Zero):
-            shape = a.shape()[:-1] + b.shape()[1:]
+            shape = ash[:-1] + bsh[1:]
             free_indices, index_dimensions = merge_indices(a, b)
             return Zero(shape, free_indices, index_dimensions)
-        if ar == 0 and br == 0:
+        if scalar: # TODO: Move this to def dot()?
             return a * b
         return CompoundTensorOperator.__new__(cls)
 
@@ -236,22 +248,22 @@ class Dot(CompoundTensorOperator):
         self._a = a
         self._b = b
         self._free_indices, self._index_dimensions = merge_indices(a, b)
-    
+
     def operands(self):
         return (self._a, self._b)
-    
+
     def free_indices(self):
         return self._free_indices
-    
+
     def index_dimensions(self):
         return self._index_dimensions
-    
+
     def shape(self):
         return self._a.shape()[:-1] + self._b.shape()[1:]
 
     def __str__(self):
         return "%s . %s" % (parstr(self._a, self), parstr(self._b, self))
-    
+
     def __repr__(self):
         return "Dot(%r, %r)" % (self._a, self._b)
 
@@ -274,22 +286,22 @@ class Cross(CompoundTensorOperator):
         self._a = a
         self._b = b
         self._free_indices, self._index_dimensions = merge_indices(a, b)
-    
+
     def operands(self):
         return (self._a, self._b)
-    
+
     def free_indices(self):
         return self._free_indices
-    
+
     def index_dimensions(self):
         return self._index_dimensions
-    
+
     def shape(self):
         return (3,)
 
     def __str__(self):
         return "%s x %s" % (parstr(self._a, self), parstr(self._b, self))
-    
+
     def __repr__(self):
         return "Cross(%r, %r)" % (self._a, self._b)
 
@@ -305,22 +317,22 @@ class Trace(CompoundTensorOperator):
     def __init__(self, A):
         CompoundTensorOperator.__init__(self)
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return self._A.free_indices()
-    
+
     def index_dimensions(self):
         return self._A.index_dimensions()
-    
+
     def shape(self):
         return ()
-    
+
     def __str__(self):
         return "tr(%s)" % self._A
-    
+
     def __repr__(self):
         return "Trace(%r)" % self._A
 
@@ -345,22 +357,22 @@ class Determinant(CompoundTensorOperator):
     def __init__(self, A):
         CompoundTensorOperator.__init__(self)
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return ()
-    
+
     def index_dimensions(self):
-        return {}
-    
+        return EmptyDict
+
     def shape(self):
         return ()
-    
+
     def __str__(self):
         return "det(%s)" % self._A
-    
+
     def __repr__(self):
         return "Determinant(%r)" % self._A
 
@@ -388,22 +400,22 @@ class Inverse(CompoundTensorOperator):
     def __init__(self, A):
         CompoundTensorOperator.__init__(self)
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return ()
-    
+
     def index_dimensions(self):
-        return {}
-    
+        return EmptyDict
+
     def shape(self):
         return self._A.shape()
-    
+
     def __str__(self):
         return "%s^-1" % parstr(self._A, self)
-    
+
     def __repr__(self):
         return "Inverse(%r)" % self._A
 
@@ -414,26 +426,27 @@ class Cofactor(CompoundTensorOperator):
         CompoundTensorOperator.__init__(self)
         sh = A.shape()
         ufl_assert(len(sh) == 2, "Cofactor of tensor with rank != 2 is undefined.")
-        ufl_assert(sh[0] == sh[1], "Cannot take cofactor of rectangular matrix with dimensions %s." % repr(sh))
+        if sh[0] != sh[1]:
+            error("Cannot take cofactor of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Cofactor.")
         ufl_assert(not isinstance(A, Zero), "Cannot take cofactor of zero matrix.")
         self._A = A
 
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return ()
-    
+
     def index_dimensions(self):
-        return {}
-    
+        return EmptyDict
+
     def shape(self):
         return self._A.shape()
 
     def __str__(self):
         return "cofactor(%s)" % self._A
-    
+
     def __repr__(self):
         return "Cofactor(%r)" % self._A
 
@@ -443,8 +456,8 @@ class Deviatoric(CompoundTensorOperator):
     def __new__(cls, A):
         sh = A.shape()
         ufl_assert(len(sh) == 2, "Deviatoric part of tensor with rank != 2 is undefined.")
-        ufl_assert(sh[0] == sh[1],
-            "Cannot take deviatoric part of rectangular matrix with dimensions %s." % repr(sh))
+        if sh[0] != sh[1]:
+            error("Cannot take deviatoric part of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Deviatoric.")
         if isinstance(A, Zero):
             return Zero(A.shape(), A.free_indices(), A.index_dimensions())
@@ -456,19 +469,19 @@ class Deviatoric(CompoundTensorOperator):
 
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return self._A.free_indices()
-    
+
     def index_dimensions(self):
         return self._A.index_dimensions()
-    
+
     def shape(self):
         return self._A.shape()
-    
+
     def __str__(self):
         return "dev(%s)" % self._A
-    
+
     def __repr__(self):
         return "Deviatoric(%r)" % self._A
 
@@ -478,8 +491,8 @@ class Skew(CompoundTensorOperator):
     def __new__(cls, A):
         sh = A.shape()
         ufl_assert(len(sh) == 2, "Skew symmetric part of tensor with rank != 2 is undefined.")
-        ufl_assert(sh[0] == sh[1],
-            "Cannot take skew part of rectangular matrix with dimensions %s." % repr(sh))
+        if sh[0] != sh[1]:
+            error("Cannot take skew part of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Skew.")
         if isinstance(A, Zero):
             return Zero(A.shape(), A.free_indices(), A.index_dimensions())
@@ -488,22 +501,22 @@ class Skew(CompoundTensorOperator):
     def __init__(self, A):
         CompoundTensorOperator.__init__(self)
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return self._A.free_indices()
-    
+
     def index_dimensions(self):
         return self._A.index_dimensions()
-    
+
     def shape(self):
         return self._A.shape()
-    
+
     def __str__(self):
         return "skew(%s)" % self._A
-    
+
     def __repr__(self):
         return "Skew(%r)" % self._A
 
@@ -513,8 +526,8 @@ class Sym(CompoundTensorOperator):
     def __new__(cls, A):
         sh = A.shape()
         ufl_assert(len(sh) == 2, "Symmetric part of tensor with rank != 2 is undefined.")
-        ufl_assert(sh[0] == sh[1],
-            "Cannot take symmetric part of rectangular matrix with dimensions %s." % repr(sh))
+        if sh[0] != sh[1]:
+            error("Cannot take symmetric part of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Sym.")
         if isinstance(A, Zero):
             return Zero(A.shape(), A.free_indices(), A.index_dimensions())
@@ -523,21 +536,21 @@ class Sym(CompoundTensorOperator):
     def __init__(self, A):
         CompoundTensorOperator.__init__(self)
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
-    
+
     def free_indices(self):
         return self._A.free_indices()
-    
+
     def index_dimensions(self):
         return self._A.index_dimensions()
-    
+
     def shape(self):
         return self._A.shape()
-    
+
     def __str__(self):
         return "sym(%s)" % self._A
-    
+
     def __repr__(self):
         return "Sym(%r)" % self._A
