@@ -1,6 +1,6 @@
 """Classes used to group scalar expressions into expressions with rank > 0."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -11,26 +11,30 @@
 #
 # UFL is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+# along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2008-03-31
-# Last changed: 2011-04-15
+# Last changed: 2011-10-17
 
+from itertools import izip
 from ufl.log import warning, error
-from ufl.common import subdict
+from ufl.common import subdict, EmptyDict
 from ufl.assertions import ufl_assert
-from ufl.expr import Expr, WrapperType
+from ufl.expr import Expr
+from ufl.operatorbase import WrapperType
 from ufl.constantvalue import as_ufl, Zero
-from ufl.indexing import Indexed, Index, FixedIndex, MultiIndex, indices
+from ufl.indexing import Index, FixedIndex, MultiIndex, indices
+from ufl.indexed import Indexed
 
 # --- Classes representing tensors of UFL expressions ---
 
 class ListTensor(WrapperType):
-    __slots__ = ("_expressions", "_free_indices", "_shape", "_repr")
+    """UFL operator type: Wraps a list of expressions into a tensor valued expression of one higher rank."""
+    __slots__ = ("_expressions", "_free_indices", "_shape")
 
     def __new__(cls, *expressions):
         # All lists and tuples should already be unwrapped in as_tensor
@@ -52,7 +56,7 @@ class ListTensor(WrapperType):
         # through the listtensor concept and free indices.
         # Are there any cases where it makes sense to even
         # have any free indices here?
-        if any(fi != e.free_indices() for e in expressions):
+        if any(set(fi) - set(e.free_indices()) for e in expressions):
             error("ListTensor assumption 2 failed, "\
                   "please report this incident as a potential bug.")
         if any(idim != e.index_dimensions() for e in expressions):
@@ -77,7 +81,9 @@ class ListTensor(WrapperType):
         ufl_assert(all(not (indexset ^ set(e.free_indices())) for e in self._expressions),\
             "Can't combine subtensor expressions with different sets of free indices.")
 
-        self._repr = "ListTensor(%s)" % ", ".join(repr(e) for e in self._expressions)
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return all(e.is_cellwise_constant() for e in self.operands())
 
     def operands(self):
         return self._expressions
@@ -91,14 +97,16 @@ class ListTensor(WrapperType):
     def shape(self):
         return self._shape
 
-    def evaluate(self, x, mapping, component, index_values):
+    def evaluate(self, x, mapping, component, index_values, derivatives=()):
         ufl_assert(len(component) == len(self._shape),
                    "Can only evaluate scalars, expecting a component "\
                    "tuple of length %d, not %s." % (len(self._shape), component))
         a = self._expressions[component[0]]
         component = component[1:]
-        a = a.evaluate(x, mapping, component, index_values)
-        return a
+        if derivatives:
+            return a.evaluate(x, mapping, component, index_values, derivatives)
+        else:
+            return a.evaluate(x, mapping, component, index_values)
 
     def __getitem__(self, key):
         origkey = key
@@ -113,7 +121,7 @@ class ListTensor(WrapperType):
             return sub if len(key) == 1 else sub[key[1:]]
 
         return Expr.__getitem__(self, origkey)
-    
+
     def __str__(self):
         def substring(expressions, indent):
             ind = " "*indent
@@ -132,11 +140,12 @@ class ListTensor(WrapperType):
         return substring(self._expressions, 0)
 
     def __repr__(self):
-        return self._repr
+        return "ListTensor(%s)" % ", ".join(repr(e) for e in self._expressions)
 
 class ComponentTensor(WrapperType):
+    """UFL operator type: Maps the free indices of a scalar valued expression to tensor axes."""
     __slots__ = ("_expression", "_indices", "_free_indices",
-                 "_index_dimensions", "_shape", "_str", "_repr")
+                 "_index_dimensions", "_shape")
 
     def __new__(cls, expression, indices):
         if isinstance(expression, Zero):
@@ -156,16 +165,16 @@ class ComponentTensor(WrapperType):
         ufl_assert(isinstance(expression, Expr), "Expecting ufl expression.")
         ufl_assert(expression.shape() == (), "Expecting scalar valued expression.")
         self._expression = expression
-        
+
         ufl_assert(all(isinstance(i, Index) for i in indices),
            "Expecting sequence of Index objects, not %s." % repr(indices))
-        
+
         dims = expression.index_dimensions()
-        
+
         if not isinstance(indices, MultiIndex): # if constructed from repr
             indices = MultiIndex(indices, subdict(dims, indices))
         self._indices = indices
-        
+
         eset = set(expression.free_indices())
         iset = set(self._indices)
         freeset = eset - iset
@@ -174,46 +183,58 @@ class ComponentTensor(WrapperType):
         missingset = iset - eset
         if missingset:
             error("Missing indices %s in expression %s." % (missingset, expression))
-        
-        self._index_dimensions = dict((i, dims[i]) for i in self._free_indices)
-        
+
+        self._index_dimensions = dict((i, dims[i]) for i in self._free_indices) or EmptyDict
         self._shape = tuple(dims[i] for i in self._indices)
 
-        self._str = "{ A | A_{%s} = %s }" % (self._indices, self._expression)
-        self._repr = "ComponentTensor(%r, %r)" % (self._expression, self._indices)
-    
+    def is_cellwise_constant(self):
+        "Return whether this expression is spatially constant over each cell."
+        return self._expression.is_cellwise_constant()
+
+    def reconstruct(self, expressions, indices):
+        # Special case for simplification as_tensor(A[ii], ii) -> A
+        if isinstance(expressions, Indexed):
+            A, ii = expressions.operands()
+            if indices == ii:
+                #print "RETURNING", A, "FROM", expressions, indices, "SELF IS", self
+                return A
+        return WrapperType.reconstruct(self, expressions, indices)
+
     def operands(self):
         return (self._expression, self._indices)
-    
+
     def free_indices(self):
         return self._free_indices
-    
+
     def index_dimensions(self):
         return self._index_dimensions
-    
+
     def shape(self):
         return self._shape
-    
+
     def evaluate(self, x, mapping, component, index_values):
         indices = self._indices
         a = self._expression
-        
+
+        ufl_assert(len(indices) == len(component),
+                   "Expecting a component matching the indices tuple.")
+
         # Map component to indices
-        for i, c in zip(indices, component):
+        for i, c in izip(indices, component):
             index_values.push(i, c)
-        
+
         a = a.evaluate(x, mapping, (), index_values)
-        
+
         for _ in component:
             index_values.pop()
-        
+
         return a
 
     def __str__(self):
-        return self._str
-    
+        return "{ A | A_{%s} = %s }" % (self._indices, self._expression)
+
     def __repr__(self):
-        return self._repr
+        return "ComponentTensor(%r, %r)" % (self._expression, self._indices)
 
 # --- User-level functions to wrap expressions in the correct way ---
 
@@ -223,21 +244,9 @@ def numpy2nestedlists(arr):
         return arr
     return [numpy2nestedlists(arr[k]) for k in range(arr.shape[0])]
 
-def tmp():
-    # FIXME: Should have no list or tuple here, convert those in as_tensor!
-    if isinstance(expressions[0], (list, tuple)):
-        ufl_assert(all(isinstance(e, (list, tuple)) for e in expressions),
-                   "Expecting all or no arguments being lists or tuples.")
-        expressions = [ListTensor(*sub) for sub in expressions]
-
-    if not all(isinstance(e, ListTensor) for e in expressions):
-        expressions = [as_ufl(e) for e in expressions]
-        ufl_assert(all(isinstance(e, Expr) for e in expressions), \
-                   "Expecting list of subtensors or expressions.")
-
 def _as_list_tensor(expressions):
     if isinstance(expressions, (list, tuple)):
-        expressions = [_as_list_tensor(e) for e in expressions] 
+        expressions = [_as_list_tensor(e) for e in expressions]
         return ListTensor(*expressions)
     else:
         return as_ufl(expressions)
@@ -252,8 +261,27 @@ def from_numpy_to_lists(expressions):
     return expressions
 
 def as_tensor(expressions, indices = None):
-    #"""COMMENTME."""
+    """UFL operator: Make a tensor valued expression.
+
+    This works in two different ways, by using indices or lists.
+
+    1) Returns A such that A[indices] = expressions.
+    If indices are provided, expressions must be a scalar
+    valued expression with all the provided indices among
+    its free indices. This operator will then map each of these
+    indices to a tensor axis, thereby making a tensor valued
+    expression from a scalar valued expression with free indices.
+
+    2) Returns A such that A[k,...] = expressions[k].
+    If no indices are provided, expressions must be a list
+    or tuple of expressions. The expressions can also consist
+    of recursively nested lists to build higher rank tensors.
+    """
     if indices is None:
+        # Allow as_tensor(as_tensor(A)) and as_vector(as_vector(v)) in user code
+        if isinstance(expressions, Expr):
+            return expressions
+
         # Support numpy array, but avoid importing numpy if not needed
         if not isinstance(expressions, (list, tuple)):
             expressions = from_numpy_to_lists(expressions)
@@ -285,39 +313,62 @@ def as_tensor(expressions, indices = None):
         return ComponentTensor(expressions, indices)
 
 def as_matrix(expressions, indices = None):
+    "UFL operator: As as_tensor(), but limited to rank 2 tensors."
     if indices is None:
+        # Allow as_matrix(as_matrix(A)) in user code
+        if isinstance(expressions, Expr):
+            ufl_assert(expressions.rank() == 2, "Expecting rank 2 tensor.")
+            return expressions
+
         # To avoid importing numpy unneeded, it's quite slow...
         if not isinstance(expressions, (list, tuple)):
-            try:
-                import numpy
-                if isinstance(expressions, numpy.ndarray):
-                    expressions = numpy2nestedlists(expressions)
-            except:
-                pass
+            expressions = from_numpy_to_lists(expressions)
+
+        # Check for expected list structure
         ufl_assert(isinstance(expressions, (list, tuple)),
             "Expecting nested list or tuple of Exprs.")
         ufl_assert(isinstance(expressions[0], (list, tuple)),
             "Expecting nested list or tuple of Exprs.")
-        return as_tensor(expressions)
+    else:
+        ufl_assert(len(indices) == 2, "Expecting exactly two indices.")
 
-    ufl_assert(len(indices) == 2, "Expecting exactly two indices.")
     return as_tensor(expressions, indices)
 
 def as_vector(expressions, index = None):
-    if index is not None:
-        ufl_assert(isinstance(index, Index), "Expecting Index object.")
+    "UFL operator: As as_tensor(), but limited to rank 1 tensors."
+    if index is None:
+        # Allow as_vector(as_vector(v)) in user code
+        if isinstance(expressions, Expr):
+            ufl_assert(expressions.rank() == 1, "Expecting rank 1 tensor.")
+            return expressions
+
+        # To avoid importing numpy unneeded, it's quite slow...
+        if not isinstance(expressions, (list, tuple)):
+            expressions = from_numpy_to_lists(expressions)
+
+        # Check for expected list structure
+        ufl_assert(isinstance(expressions, (list, tuple)),
+            "Expecting nested list or tuple of Exprs.")
+    else:
+        ufl_assert(isinstance(index, Index), "Expecting a single Index object.")
         index = (index,)
+
     return as_tensor(expressions, index)
 
 def as_scalar(expression):
+    """Given a scalar or tensor valued expression A, returns either of the tuples::
+
+      (a,b) = (A, ())
+      (a,b) = (A[indices], indices)
+
+    such that a is always a scalar valued expression."""
     ii = indices(expression.rank())
     if ii:
-        #mi = MultiIndex(ii, expression.shape())
         expression = expression[ii]
     return expression, ii
 
 def relabel(A, indexmap):
-    "Relabel free indices of A with new indices, using the given mapping."
+    "UFL operator: Relabel free indices of A with new indices, using the given mapping."
     ii = tuple(sorted(indexmap.keys()))
     jj = tuple(indexmap[i] for i in ii)
     ufl_assert(all(isinstance(i, Index) for i in ii), "Expecting Index objects.")
@@ -333,19 +384,25 @@ def unit_list2(i, j, n):
     return [[(1 if (i == i0 and j == j0) else 0) for j0 in xrange(n)] for i0 in xrange(n)]
 
 def unit_vector(i, d):
+    "UFL value: A constant unit vector in direction i with dimension d."
     return as_vector(unit_list(i, d))
 
 def unit_vectors(d):
+    "UFL value: A tuple of constant unit vectors in all directions with dimension d."
     return tuple(unit_vector(i, d) for i in range(d))
 
 def unit_matrix(i, j, d):
+    "UFL value: A constant unit matrix in direction i,j with dimension d."
     return as_matrix(unit_list2(i, j, d))
 
 def unit_matrices(d):
+    "UFL value: A tuple of constant unit matrices in all directions with dimension d."
     return tuple(unit_matrix(i, j, d) for i in range(d) for j in range(d))
 
 def dyad(d, *iota):
     "TODO: Develop this concept, can e.g. write A[i,j]*dyad(j,i) for the transpose."
+    from ufl.constantvalue import Identity
+    from ufl.operators import outer # a bit of circular dependency issue here
     I = Identity(d)
     i = iota[0]
     e = as_vector(I[i,:], i)
@@ -353,23 +410,34 @@ def dyad(d, *iota):
         e = outer(e, as_vector(I[i,:], i))
     return e
 
-def _test():
-    #from ufl.tensors import unit_vector, unit_vectors, unit_matrix, unit_matrices
-    from ufl.objects import triangle
-    cell = triangle
-    d = cell.geometric_dimension()
-    ei, ej, ek = unit_vectors(d)
-    eii, eij, eik, eji, ejj, ejk, eki, ekj, ekk = unit_matrices(d)
-    print ei
-    print ej
-    print ek
-    print eii
-    print eij
-    print eik
-    print eji
-    print ejj
-    print ejk
-    print eki
-    print ekj
-    print ekk
+def unit_indexed_tensor(shape, component):
+    from ufl.constantvalue import Identity
+    from ufl.operators import outer # a bit of circular dependency issue here
+    r = len(shape)
+    if r == 0:
+        return 0, ()
+    jj = indices(r)
+    es = []
+    for i in xrange(r):
+        s = shape[i]
+        c = component[i]
+        j = jj[i]
+        e = Identity(s)[c,j]
+        es.append(e)
+    E = es[0]
+    for e in es[1:]:
+        E = outer(E, e)
+    return E, jj
 
+def unwrap_list_tensor(lt):
+    components = []
+    sh = lt.shape()
+    subs = lt.operands()
+    if len(sh) == 1:
+        for s in xrange(sh[0]):
+            components.append(((s,),subs[s]))
+    else:
+        for s,sub in enumerate(subs):
+            for c,v in unwrap_list_tensor(sub):
+                components.append(((s,)+c,v))
+    return components
